local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"

local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local _M = {}

local function JSON_overlaySet()
	--TODO: check features
	local data = {}
	data.sameSource = true
	data.logo = true
	if FEATURES.DISABLE_BACKGROUND then
		data.bg = false
	else
		data.bg = true
	end

	if  FEATURES.FEATURE_MAX_RES_1080P then
		data.max_image = "1080P"
	else
		data.max_image = "4K"
	end

	MSG.OK("",{data = data})
	return true
end

local function JSON_parameter(_, args)
	local has_encode = false
	if FEATURES.FEATURE_IPStream or FEATURES.DEV_BT1120_VI then
		has_encode = true
	end
	local data = {}

	if args.belong == "source" then
		data.outputSet = false
		data.encodeMain = true
		data.encodeSub = true
		data.encodeOsd = true
		data.overlaySet = false
		data.encodeCropper = true
		data.videoSource = true
		data.encodeTips = true

	elseif args.belong == "output" then
		data.outputSet = true
		data.encodeMain = has_encode
		data.encodeSub = has_encode
		data.encodeOsd = true
		data.overlaySet = true
		data.encodeCropper = false
		data.videoSource = false
		data.encodeTips = has_encode

	elseif args.belong == "flow_main" then
		data.outputSet = false
		data.encodeMain = has_encode
		data.encodeSub = false
		data.encodeOsd = false
		data.overlaySet = false
		data.encodeCropper = false
		data.videoSource = false
		data.encodeTips = has_encode

	elseif args.belong == "flow_sub" then
		data.outputSet = false
		data.encodeMain = false
		data.encodeSub = has_encode
		data.encodeOsd = false
		data.overlaySet = false
		data.encodeCropper = false
		data.videoSource = false
		data.encodeTips = has_encode
	end

	MSG.OK("",{data = data})
	return true
end

local function JSON_pattern()
	local data = {
					mode = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
					bgPicture = true
				}
	
	if FEATURES.DISABLE_BACKGROUND then
		data.bgPicture = false
	end

	if FEATURES.FEATURE_MAX_SUPPORT_SCREEN == 1 then
		data.mode = {2}
	elseif 	FEATURES.FEATURE_MAX_SUPPORT_SCREEN == 4 then
		data.mode = {1, 2, 3, 4, 5}
	end

	MSG.OK("",{data = data})
	return true
end

local function JSON_sourceType()
	local data = {
					rtsp = true,
					rtmp = true,
					rtmps = true,
					rtp = true,
					udp = true,
					http = true,
					https = true,
					srt = true,
					ndi = true,
					file = false,
					rtmp_server = true
	}

	if FEATURES.FEATURE_DISABLE_NDI then
		data.ndi = false
	end

	if FEATURES.FEATURE_PLAYBACK then
		data.file = true
	end

	MSG.OK("",{data = data})	
	return true
end

local function JSON_streamerDebug()
	if FEATURES.FEATURE_LOCAL_STREAM then
		ngx.say([[
		{
			"result": "ok",
			"data": {
				"debug": true
			}
		}
		]])
	else
		ngx.say([[
			{
				"result": "ok",
				"data": {
					"debug": false
				}
			}
			]])
	end
	return true
end

local function JSON_remixConfig()
	if FEATURES.FEATURE_LOCAL_STREAM then
		ngx.say([[
		{
			"result": "ok",
			"data": {
				"lineGain": true,
				"localOutput":true
			}
		}
		]])
	else
		ngx.say([[
			{
				"result": "ok",
				"data": {
					"lineGain": false,
					"localOutput":false
				}
			}
			]])
	end
	return true
end


local function JSON_discoveryConfig()
	if FEATURES.DEVICE_TYPE == "E_DOCTOR_S" then
		ngx.say([[
		{
			"result": "ok",
			"data": {
				"addTerminal": true,
				"authCode": true
			}
		}
		]])
	else
		ngx.say([[
			{
				"result": "ok",
				"data": {
					"addTerminal": false,
					"authCode": false
				}
			}
			]])
	end
	return true
end

_M.APIS = _M.APIS or {}
_M.APIS["overlaySet"] = JSON_overlaySet
_M.APIS["parameter"] = JSON_parameter
_M.APIS["pattern"] = JSON_pattern
_M.APIS["sourceType"] = JSON_sourceType
_M.APIS["streamerDebug"] = JSON_streamerDebug
_M.APIS["remixConfig"] = JSON_remixConfig
_M.APIS["discovery"] = JSON_discoveryConfig

return _M
