local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"

local dset = {}
--------------------------------------
local function API_enum_ip(red, args)
	local proxy = CO.GetProxy( "/", "networkmanager" )
	if not proxy then
		MSG.ERROR( "networkError" )
		return true
	end

	local r, allIntfs = proxy.GetAllInterfaces()
	if not r or not allIntfs then
		MSG.ERROR( "getInterfaceError" )
		return true
	end

	proxy:Destroy()

	local rtable = {}
	local item
	local friendName
	for r,item in pairs(allIntfs) do
		if item.family == "AF_INET" and item.name ~= "lo" and item.address ~= "10.254.254.1" then
			local ifn, als = item.name:match("(.+):(%d*)")
			if not ifn then
				ifn = item.name
			end
			if ifn == "eth0" then
				friendName = "网卡1"
			elseif ifn == "eth1" then
				friendName = "网卡2"
			else
				friendName = "其他网卡"
			end
			
			if als then
				als = tonumber(als)
				if als then
					als = als + 2
				else
					als = 2
				end
				friendName = friendName .. ":地址" .. als
			end

			friendName = friendName .. " (" .. item.address .. ")"

			table.insert(rtable, {item.name, friendName})
		end
	end

	MSG.OK("", { data = rtable })
	return true
end

local function API_get(red, args)
	local proxy = CO.GetProxy( "/", "networkmanager" )
	local cfg_proxy = CO.GetProxy( "/config/nm", "networkmanager" )
	if not proxy or not cfg_proxy then
		MSG.ERROR( "networkError" )
		return true
	end

	local r
	local eth0, eth1
	r, eth0 = proxy.GetLinkDetail("eth0")
	if not r then
		MSG.ERROR( "geteth0Error" )
		return true
	end

	r, eth1 = proxy.GetLinkDetail("eth1")
	if not r then
		MSG.ERROR( "geteth1Error" )
		return true
	end

	proxy:Destroy()

	local aliases = cfg_proxy._P("aliases")
	cfg_proxy:Destroy()

	local alias_eth0 = aliases and aliases.eth0 or {}
	local alias_eth1 = aliases and aliases.eth1 or {}
	local retData={
		{
			device = "eth0",
			state = eth0.status,
			ip = eth0.address,
			mask = eth0.netmask,
			mac = eth0.mac,
			gw = eth0.gw,
			dynamic = eth0.method == "dhcp" and "y" or "n",
			dns = eth0.dns and eth0.dns[1] or "",

			ip2 = alias_eth0.address or "0.0.0.0",
			mask2 = alias_eth0.netmask or "255.255.255.0",
			gw2 = alias_eth0.gw or "",

			ip3 = alias_eth0.address2 or "0.0.0.0",
			mask3 = alias_eth0.netmask2 or "255.255.255.0",
			gw3 = alias_eth0.gw2 or "",
			enable = alias_eth0.enable and 1 or 0,
		},
		{
			device = "eth1",
			state = eth1.status,
			ip = eth1.address,
			mask = eth1.netmask,
			mac = eth1.mac,
			gw = eth1.gw,
			dynamic = eth1.method == "dhcp" and "y" or "n",
			dns = eth1.dns and eth1.dns[1] or "",

			ip2 = alias_eth1.address or "0.0.0.0",
			mask2 = alias_eth1.netmask or "255.255.255.0",
			gw2 = alias_eth1.gw or "",

			ip3 = alias_eth1.address2 or "0.0.0.0",
			mask3 = alias_eth1.netmask2 or "255.255.255.0",
			gw3 = alias_eth1.gw2 or "",
			enable = alias_eth1.enable and 1 or 0,
		}
	}

	MSG.OK( "", { data = retData } )
	return true
end

--------------------------------------

local function API_modify(red, args)
	if args and args.device then
		local proxy = CO.GetProxy( "/config/nm", "networkmanager" )
		if not proxy then
			MSG.ERROR( "networkError" )
			return true
		end
		if tonumber(args.enable) == 0 then
			args.enable = false
		else
			args.enable = true
		end
		
		local sets = {
			["ethernets." .. args.device .. ".address"] = args.ip,
			["ethernets." .. args.device .. ".netmask"] = args.mask,
			["ethernets." .. args.device .. ".mac"] = args.mac,
			["ethernets." .. args.device .. ".gw"] = args.gw,
			["ethernets." .. args.device .. ".method"] = args.dynamic and (args.dynamic == "y" and "dhcp" or "static") or nil,
			["ethernets." .. args.device .. ".dns[1]" ] = args.dns,

			["aliases." .. args.device .. ".enable"] = args.enable,

			["aliases." .. args.device .. ".address" ] = args.ip2,
			["aliases." .. args.device .. ".netmask" ] = args.mask2,
			["aliases." .. args.device .. ".gw" ] = args.gw2,
			["aliases." .. args.device .. ".address2" ] = args.ip3,
			["aliases." .. args.device .. ".netmask2" ] = args.mask3,
			["aliases." .. args.device .. ".gw2" ] = args.gw3
		}

		local r,rr = proxy:P_SET(sets)
		if not r or not rr then
			MSG.ERROR( "setNetworkError" )
		else
			MSG.OK("")
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_ping(red,args)
	if not args.ip then
		MSG.ERROR("argumentError")
	else
		local proxy =  CO.GetProxy( "/", "networkmanager","local",10 )
		if not proxy then
			MSG.ERROR( "networkError" )
			return true
		else
			local r,data,err = proxy.Ping(args.ip)
			proxy:Destroy()
			if r and data then
				MSG.OK("",{data=data})
			else
				MSG.OK("",{data={status="unreached"}})
			end
		end
		
	end
	return true
end

local function API_add_static(red, args)
	local proxy = CO.GetProxy("/static", "networkmanager")
	if not proxy then
		MSG.ERROR( "Failure to connect network management service" )
		return true
	end
	if not args.interface or not args.target or not args.netmask or not args.gateway or not args.metric or not args.mtu then
		proxy:Destroy()
		MSG.ERROR( "Missing parameter" )
		return true
	end
	details = {
		interface = args.interface,
		target = args.target,
		netmask = args.netmask,
		gateway = args.gateway,
		metric = args.metric,
		mtu = args.mtu
	}
	local is_ok, result,err = proxy.AddStatic(details)
	proxy:Destroy()
	if not is_ok or not result then
		MSG.ERROR( err or "Failure to add static route" )
		return true
	end
	MSG.OK("")
	return true
end

local function API_get_static(red, args)
	local proxy = CO.GetProxy("/static", "networkmanager")
	if not proxy then
		MSG.ERROR( "Failure to connect network management service" )
		return true
	end
	local is_ok, result,err = proxy.GetStatic()
	proxy:Destroy()
	if not is_ok or not result then
		MSG.ERROR( err or "Failure to get static route" )
		return true
	end
	MSG.OK("",{data=result})
	return true
end

local function API_modify_static(red, args)
	local proxy = CO.GetProxy("/static", "networkmanager")
	if not proxy then
		MSG.ERROR( "Failure to connect network management service" )
		return true
	end
	if not args.interface or not args.target or not args.netmask or not args.gateway or not args.id or not args.metric or not args.mtu then
		proxy:Destroy()
		MSG.ERROR( "Missing parameter" )
		return true
	end
	details = {
		interface = args.interface,
		target = args.target,
		netmask = args.netmask,
		gateway = args.gateway,
		metric = args.metric,
		mtu = args.mtu
	}
	local is_ok, result,err = proxy.ModifyStatic(args.id,details)
	proxy:Destroy()
	if not is_ok or not result then
		MSG.ERROR( err or "Failure to modify static route" )
		return true
	end
	MSG.OK("")
	return true
end

local function API_remove_static(red, args)
	local proxy = CO.GetProxy("/static", "networkmanager")
	if not proxy then
		MSG.ERROR( "Failure to connect network management service" )
		return true
	end
	if not args or not args.ids then
		proxy:Destroy()
		MSG.ERROR("Invalid parameters" )
		return true
	end
	local is_ok, result,err = proxy.RemoveStatic(args.ids)
	if not is_ok or not result then
		MSG.ERROR(err or "Failure to del static route" )
		return true
	end
	MSG.OK("")
	return true
end

local function API_set_static(red, args)
	local proxy = CO.GetProxy("/static", "networkmanager")
	if not proxy then
		MSG.ERROR( "Failure to connect network management service" )
		return true
	end
	if not args or not args.id or not args.status then
		proxy:Destroy()
		MSG.ERROR( "Invalid parameters" )
		return true
	end
	local is_ok, result,err = proxy.SetStatic(args.id,args.status)
	if not is_ok or not result then
		MSG.ERROR( err or "Failure to set static route" )
		return true
	end
	MSG.OK("")
	return true
end

local function API_set_hostname(red, args)
	if args and args.hostname then
		local proxy = CO.GetProxy( "/", "networkmanager" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,done = proxy.SetHostName(args.hostname)
			if not r or not done then
				proxy:Destroy()
				MSG.ERROR( "setHostNameError" )
				return true;
			end
		end

		proxy = CO.GetProxy( "/streamer", "codec" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r,done = proxy.UpdateAllNDIService()
			if not r or not done then
				proxy:Destroy()
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_get_hostname(red, args)
	local proxy = CO.GetProxy( "/", "networkmanager" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r,ret = proxy.GetHostName()
		if not r or not ret then
			proxy:Destroy()
			MSG.ERROR( "getHostNameError" )
		else
			local result = {
				hostname = ret
			}
			MSG.OK("", {data = result})
		end
	end
	return true
end

--------------------------------------
dset.APIS = dset.APIS or {}
dset.APIS["ping"] = API_ping

dset.APIS["get"] = API_get
dset.APIS["modify"] = API_modify
dset.APIS["enum_ip"] = API_enum_ip
dset.APIS["addStatic"] = API_add_static
dset.APIS["getStatic"] = API_get_static
dset.APIS["removeStatic"] = API_remove_static
dset.APIS["modifyStatic"] = API_modify_static
dset.APIS["setStatic"] = API_set_static
dset.APIS["set_hostname"] = API_set_hostname
dset.APIS["get_hostname"] = API_get_hostname
--------------------------------------
return dset

