local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}
--------------------------------------
local function API_get(red, args)
	-- local out = 1
	-- if args and args.output then
	-- 	out = tonumber(args.output) or 1
	-- end
	local out = tonumber(args.output)
	local BGimgUrl = ""
	local bg_enable = 0
	local name = ""
	
	if out==0 then
		name = "/preview"
	else
		name = "/output/hd" .. (out-1)
		local proxy_img = CO.GetProxy( "/pics", "picManager","local",15 )
		if not proxy_img then
			MSG.ERROR( "picManagerError" )
			return true
		else
			local r, err
			r,BGimgUrl,err = proxy_img.GetBGPic(out)
			proxy_img:ChangeObject("/relation")
			r,bg_enable,err = proxy_img.getHideInfo(out,"bg")
			bg_enable = tonumber(bg_enable[1].enable) or 0
			proxy_img:Destroy()

		end
	end
	local proxy = CO.GetProxy( name, "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local nolayout
		nolayout = tonumber(args.no_layout)
		if nolayout then
			nolayout = nolayout ~= 0
		end
		local r, status,err = proxy.GetStatus(nolayout)
		proxy:Destroy()

		if not r or not status then
			MSG.ERROR( "getStatusError",err )
			return true
		end

		local result = {
			layout_id = status.layout or 1,
			--bordercolor  = status.border.bordercolor ,
			--bordercolor_enabled   = status.border.enabled ,
			layout_number = status.layout_number or 1,
			output = status.started and "on" or "off",
			resolution = status.resolution or "1080P30",
			resolution_name = status.resolution_name or status.resolution or "1080P30",
			sampling = status.sampling or 48000,
			hdmi_mode = status.hdmi_mode or "AUTO",
			colorspace = status.colorspace or "AUTO",
			BGimgUrl = BGimgUrl,
			bg_enable = bg_enable,
			
		}
		if not status.border and status.border~="" then
			result.bordercolor_enabled = status.border.enabled
			result.bordercolor  = status.border.bordercolor
		else
			result.bordercolor_enabled = false
			result.bordercolor  = 0
		end

		if status.channels then
			result.layout = {}	

			proxy = CO.GetProxy( "/streams", "codec" )
			if not proxy then
				_dbg.Error( "OUTPUT", "Warning: Fail to get codec:/streams proxy so could not get stream details!" )
			end
			local i, item
			for i=1, #status.channels do
				item = {
					id = status.channels[i].id,
					stream_id = status.channels[i].stream,
					status = status.channels[i].bind and "on" or "off",
					stream_name = "未知",
					volume = "off",
					src = "UNKNOWN",
					manual_url = "",
					bitrate = 0,
					resolution = "分辨率未知",
					buffer = "live:200:200:500:0:0"
				}

				if proxy then
					local detail
					r, detail = proxy.GetStreamDetail( status.channels[i].stream )

					if r and detail then
						item.buffer      = detail.buffer_option or item.buffer_option
						item.stream_name = detail.name
						item.source      = detail.source or 1
						item.type        = detail.type 
						item.origin      = detail.origin 
						item.src         = detail.address or "UNKNOWN"
						item.manual_url  = detail.manual_url or "UNKNOWN"
						local originIp,imgUrl
						if detail.origin =="remote" then
							local num = string.find(detail.imgUrl,"%/")
							originIp  = string.sub(detail.imgUrl,1,num-1)
							imgUrl    = string.sub(detail.imgUrl,num)
						else
							imgUrl = detail.imgUrl
						end

						item.imgUrl   = imgUrl 
						item.originIp = originIp 

						item.volume = detail.audio_enables and detail.audio_enables[ "hd" .. (out-1) ] and "on" or "off"
						if detail.stream_info then
							item.bitrate = detail.stream_info.bitrate or 0
							item.resolution = detail.stream_info.resolution or "分辨率未知"

							if detail.stream_info.live == "connected" then
								if detail.stream_info.stream_start and detail.stream_info.output_start then
									item.status = "view_pull"
								elseif detail.stream_info.stream_start then
									item.status = "pull"
								elseif detail.stream_info.output_start then
									item.status = "view"
								end
							else
								if detail.stream_info.live ~= "closed" and detail.stream_info.live ~= "" then
									item.status = detail.stream_info.live
								else
									item.status = "off"
								end
							end
						end

						if detail.srt_status then
							item.srtStatus = detail.srt_status
						end

						if detail.rtmp_server_status then
							item.rtmp_server_status = detail.rtmp_server_status
						end
					end
				end

				table.insert( result.layout, item )
			end

			if proxy then
				proxy:Destroy()
			end
		end

		MSG.OK( "", {
			data = result
		})

		return true
	end
end

--------------------------------------
local function API_audio(red, args)
	local out = 1
	if args and args.output then
		out = tonumber(args.output) or 1
	end

	local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, analog_dis, analog_gain, analog_local_en = proxy.GetAudioGain("analog")
		if not r then
			analog_dis = 1
			analog_gain = 0
			analog_local_en = 0
		end
		local r, emb_dis, emb_gain, emb_local_en = proxy.GetAudioGain("embedded")
		if not r then
			emb_dis = 1
			emb_gain = 0
			emb_local_en = 0
		end
		proxy:Destroy()

		local result = {
			audio_analog_gain = {
				value = analog_gain,
				disable = analog_dis,
				localOutput = analog_local_en
			},
			audio_digital_gain = {
				value = emb_gain,
				disable = emb_dis,
				localOutput = emb_local_en
			}
		}

		MSG.OK( "", {
			data = result
		})

		return true
	end
end
--------------------------------------

local function API_layout(red, args)
	if args and args.action then
		-- local out = 1
		-- if args.output then
		-- 	out = tonumber(args.output) or 1
		-- end
		local out = tonumber(args.output)
		local name = ""
		if out==0 then
			name = "/preview"
		else
			name = "/output/hd" .. (out-1)
		end
		local proxy = CO.GetProxy( name, "codec", "local", 5 )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done = "无效的布局操作"
			if args.action == "remove" then
				if not args.position then
					done = "无效的窗格ID"
				else
					r, done = proxy.RemoveStreamByWindow( tonumber(args.position) )
				end
			elseif args.action == "change" then
				if not args.layout_id then
					done = "无效的布局ID"
				else
					r, done = proxy.SetOutputLayout( args.layout_id )
				end
			else
				if not args.position or not args.stream_id then
					done = "无效的窗格ID或媒体源"
				else
					r, done = proxy.BindStream( tonumber(args.position), args.stream_id )
				end
			end
			proxy:Destroy()

			if not r or not done then
				MSG.ERROR( "layoutError" )
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_set(red, args)
	if args then
		local out = 1
		if args.output then
			out = tonumber(args.output) or 1
		end
		local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done = "无效的操作"
			if args.resolution then
				r, done = proxy.SetOutputFormat( args.resolution )
				if r then
					done = true
				end
			end

			if args.hdmi_mode and args.colorspace and args.sampling then
				r, done = proxy.SetHDMIOptions( args.hdmi_mode, args.colorspace, tonumber(args.sampling) or 48000 )
				if r then
					done = true
				end
			end

			proxy:Destroy()

			if not r or not done then
				MSG.ERROR( "layoutSetError" )
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_window_audio_set(red, args)
	if args and args.volume then
		local out = 1
		if args.output then
			out = tonumber(args.output) or 1
		end
		local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done = proxy.EnableAudio( args.id or "*", args.volume == "on" and true or false )
			if args.id then
				proxy.ModifyAudioVolume(args.id, "none", args.volume == "on" and 0 or 1)
			end
			proxy:Destroy()

			if not r or not done then
				MSG.ERROR( "layoutSetError" )
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_audio_set(red, args)
	if args and args.audio then
		local out = 1
		if args.output then
			out = tonumber(args.output) or 1
		end
		local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done = "无效的操作"
			local atype = "embedded"
			if args.audio == "analog" then
				atype = "analog"
			end
			if args.disable then
				if tonumber(args.disable) == 1 then
					r, done = proxy.SetAudioGain(atype, "disable" )
				else
					r, done = proxy.SetAudioGain(atype, "enable" )
				end
			elseif args.localOutput then
				r, done = proxy.SetAudioGain( atype, "none", tonumber(args.localOutput) )
			elseif tonumber(args.value) then
				r, done = proxy.SetAudioGain( atype, tonumber(args.value) )
			else
				r, done = nil, "无效的操作"
			end

			proxy:Destroy()

			if not r or not done then
				MSG.ERROR("layoutSetError" )
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end


local function API_icon(red, args)
	if args and args.id then
		-- local out = 1
		-- if args.output then
		-- 	out = tonumber(args.output) or 1
		-- end
		local out = tonumber(args.output)
		local name = ""
		if out==0 then
			name = "/preview"
		else
			name = "/output/hd" .. (out-1)
		end
		local proxy = CO.GetProxy( name, "codec" )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local id = args.id
			if type(id) ~= "table" then
				id = { id }
			end
			local r, done,err= proxy.ChangeShowLayouts( id )
			proxy:Destroy()

			if not r or not done then
				MSG.ERROR( "layoutModifyError",err )
			else
				MSG.OK()
			end
			return true
		end
	else
		-- local out = 1
		-- if args.output then
		-- 	out = tonumber(args.output) or 1
		-- end
		local out = tonumber(args.output)
		local name = ""
		if out==0 then
			local proxy = CO.GetProxy( "/preview", "codec" )
			if not proxy then
				MSG.ERROR( "systemError" )
				return true
			else
				local data = {}
				local r, vis, icons, status
				r, icons = proxy.GetAllLayouts()
				r, vis = proxy.GetShowLayouts()
				if not r or not vis then
					vis = {"1"}
				end
				for kk,vv in pairs(vis) do
					if icons[vv] then
						icons[vv].enable = 1
					end
				end
				r, status = proxy.GetStatus(0)
				proxy:Destroy()
				if  status and status.layout then
					for k,v in pairs(icons) do
						if v.id == tonumber(status.layout) then
							v.output_used = 1
						end
					end

				end
				---调整数据类型为数组table
				for k,v in pairs(icons) do
					table.insert(data,v)
				end
			
				MSG.OK("", { data = data })
				return true
			end			
		else
			local proxy = CO.GetProxy( "/layout", "codec" )
			if not proxy then
				MSG.ERROR( "systemError" )
				return true
			else
				local data = {}
				local r, icons,err = proxy.GetAllLayouts()
				
				local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
				if not proxy then
					MSG.ERROR( "systemError" )
					return true
				else
					local r, vis = proxy.GetShowLayouts()
					if not r or not vis then
						vis = {"1"}
					end
					for kk,vv in pairs(vis) do
						if icons[vv] then
							icons[vv].enable = 1
						end
					end
					local status
					local r, status = proxy.GetStatus(out-1)
					proxy:Destroy()
					if  status and status.layout then
						for k,v in pairs(icons) do
							if v.id == tonumber(status.layout) then
								v.output_used = 1
							end
						end

					end
				end
				---调整数据类型为数组table
				for k,v in pairs(icons) do
					table.insert(data, v)
				end
			
				MSG.OK("", { data = data })
				return true
			end
		end
		
	end
end

local function API_get_port(red, args)
	local out = 1
	if args and args.output then
		out = tonumber(args.output) or 1
	end
	local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, details = proxy.GetInterfacesBinding()
		proxy:Destroy()

		if not r or not details then
			MSG.ERROR( "getOutputInterfaceError" )
		else
			MSG.OK("", {data = details})
		end
		return true
	end
end

local function API_bind_port(red, args)
	if args and args.id then
		local out = 1
		if args and args.output then
			out = tonumber(args.output) or 1
		end
		local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec", "local", 6 )
		if not proxy then
			MSG.ERROR( "systemError" )
			return true
		else
			local r, done, msg
			if not args.enable or tonumber(args.enable) == 1 then
				r, done, msg = proxy.BindVideoInterface(args.id)
			else
				r, done, msg = proxy.UnbindVideoInterface(args.id)
			end
			proxy:Destroy()
	
			if not r or not done then
				if msg then 
					MSG.ERROR( msg )
				else 
					MSG.ERROR( "setOutputPortError" )
				end
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_resolutions(red, args)
	local out = 1
	if args and args.output then
		out = tonumber(args.output) or 1
	end
	local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, details = proxy.GetSupportResolutions()
		proxy:Destroy()

		if not r or not details then
			MSG.ERROR( "getResolutionError" )
		else
			MSG.OK("", {data = details})
		end
		return true
	end
end

local function API_EnableBorder(red,args)
	if not args.bordercolor_enabled  then
		MSG.ERROR( "argumentError" )
		return true
	end
	local out = 1
	if args and args.output then
		out = tonumber(args.output) or 1
	end
	local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local tabs = {}
		tabs.bordercolor = args.bordercolor
		tabs.enabled = args.bordercolor_enabled

		local r, done,err = proxy.EnableBorder(tabs)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "setBorderError" )
		else
			MSG.OK("")
		end
		return true
	end

end

local function API_GetEnableBorder(red,args)
	if not args.output  then
		MSG.ERROR( "argumentError" )
		return true
	end
	local out = 1
	if args and args.output then
		out = tonumber(args.output) or 1
	end
	local proxy = CO.GetProxy( "/output/hd" .. (out-1), "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done,err = proxy.GetStatus(out)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "getBorderError" )
		else
			MSG.OK("",{data={bordercolor=done.border.bordercolor,bordercolor_enabled = done.border.enabled}})
		end
		return true
	end

end

--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["audio"] = API_audio
dset.APIS["layout"] = API_layout
--dset.APIS["addLayout"] = API_addLayout

dset.APIS["icon"] = API_icon
dset.APIS["audio_set"] = API_audio_set
dset.APIS["window_audio_set"] = API_window_audio_set
dset.APIS["set"] = API_set
dset.APIS["get_port"] = API_get_port
dset.APIS["bind_port"] = API_bind_port
dset.APIS["resolutions"] = API_resolutions

dset.APIS["EnableBorder"] = API_EnableBorder
dset.APIS["GetEnableBorder"] = API_GetEnableBorder

--------------------------------------
return dset

