local CO = require "api_coservice"
local MSG = require "api_msg"
local cjson = require "cjson"

local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local dset = {}

local function API_get(red, args)
    local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","output"})
		proxy:Destroy()

		if not r or not streams then
			MSG.ERROR( "getStreamsError" )
			return true
		end

		local result = {
		}

		local i, item
		for i=1, #streams do
            if not args or not args.id or args.id == streams[i].id then
                ---TODO 判断类型为local
                if not streams[i].sourceType or streams[i].sourceType=="main" then
                    item = {
                        id = streams[i].id,
                        name = streams[i].name,
                        type = streams[i].type,
                        sourceType = streams[i].sourceType
                    }
                    ---文件状态
                    local file_proxy = CO.GetProxy( "/record", "codec" )
                    if not file_proxy then
                        MSG.ERROR( "systemError" )
                        return true
                    else
                        local r,Data,err = file_proxy.GetChannelRecordStatus(streams[i].id)
                        file_proxy:Destroy()
                        if r and  Data then
                            if not Data.record_info then
                                Data.record_info={}
                            end
                            item.startTime = Data.record_info.startTime or ""
                            item.work = Data.work or ""
                            item.stopTime = Data.record_info.stopTime or ""
                            item.totalSize = Data.record_info.totalSize or ""
                            item.totalDuration = Data.record_info.totalDuration or ""
                        end
                    end
                end
				table.insert( result, item )
			end
        end     
		MSG.OK( "", {
			data = result
		})

		return true
	end
    
end


--查看录像文件
local function API_record_file(red, args)
    if not args.id then
        MSG.ERROR( "argumentError" )
        return true
    end

    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
		MSG.ERROR( "systemError" )
    else
        if FEATURES.DEVICE_TYPE == "D300" then
            -- local r,Data,err = proxy.GetSysFileRecordList(args.id)
            local r,Data,err = proxy.GetSysFileRecordList(1, args.id)
            proxy:Destroy()
            if not r or not Data then
                -- MSG.ERROR( "查看录像文件失败！错误:"..err )
                MSG.ERROR("GetSysFileRecordList Error")
            else
                local data = Data[0] and Data[0].list
                MSG.OK( "", {
                    data= data or {},
                    code = 0,
                    count = 1000,
                })
            end
        else
            local r,Data,err = proxy.GetChannelRecordList(args.id)
            proxy:Destroy()
            if not r or not Data then
                -- MSG.ERROR( "查看录像文件失败！错误:"..err )
                MSG.ERROR("getRecordListError")
            else
                local data
                local path 
            
                if tonumber(args.diskId) == 2 then
                    data = Data[2] and Data[2].list
                    if Data[2].path then
                        path = Data[2].path
                    end
                else
                    data =  Data[1].list or {}
                    if Data[1].path then
                        path = Data[1].path
                    end
                end
                if type(data) =="table" then
                    for k,v in pairs(data) do
                        if v.id then
                            v.url = path .. "/" .. v.id
                        end
                    end
                end
                MSG.OK("",{
                    data= data or {},
                    code = 0,
                    count = 1000,
                })
            end
        end
    end
  
    return true
end

--查看录像配置
local function API_getconf(red, args)
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.GetRecordOptions()
        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "查询录像参数失败！错误:"..err )
            MSG.ERROR("getRecordOptionsError")
        else
            MSG.OK("",{
                data=Data,
                code = 0,
                count = 1000,
            })
        end
    end
    return true
end

--启动所有录制
local function API_start(red,args) 
    if  not args.id then
        MSG.ERROR( "argumentError" )
        return true
    end 
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.StartChannel(args.id)

        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "启动录制失败！错误:"..err )
            MSG.ERROR("startRecordError")
        else
            MSG.OK("")
        end
    end
    return true
end


local function API_stopAll(red,args)
   
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.StopRecord()

        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "停止所有录制失败！错误:"..err )
            MSG.ERROR("stopRecordError")
        else
            MSG.OK("")
        end
    end
    return true
end


local function API_startAll(red,args)   

    local err_info = ""
    local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","output"})
		proxy:Destroy()

		if not r or not streams then
			MSG.ERROR( "getStreamsError" )
			return true
		end

		local i
		for i=1, #streams do
            if not args or not args.id or args.id == streams[i].id then
                ---TODO 判断类型为local
                if not streams[i].sourceType or streams[i].sourceType=="main" then
                    local proxy = CO.GetProxy( "/record", "codec" )
                    if not proxy then
                        MSG.ERROR( "systemError" )
                    else
                        local r,Data,err = proxy.StartChannel(streams[i].id)
                        proxy:Destroy()
                        if not r or not Data then
                            err_info = err_info .."(流：".. streams[i].id ..",错误：".. err .."!) "
                        end
                    end
                end
			end
        end
    end
    if err_info ~= "" then
        -- MSG.ERROR( "录制启动失败！详细错误信息:" .. err_info)
        MSG.ERROR("startAllRecordError")
    else
        MSG.OK("")
    end  
    
    return true
end


local function API_stop(red,args)
    if  not args.id then
        MSG.ERROR( "argumentError" )
        return true
    end
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.StopChannel(args.id)

        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "停止所有录制失败！错误:"..err )
            MSG.ERROR("stopRecordError")
        else
            MSG.OK("")
        end
    end
    return true
end

local function API_setconf(red,args)
    if not args.format or not args.limitation or not args.limitSize or not args.limitTime or not args.disk_policy or not args.disk_number then
        MSG.ERROR( "argumentError" )
        return true
    end
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "codecError" )
    else
        local tabs={}
        tabs.format =args.format
        tabs.disk_number =args.disk_number
        tabs.limitation =args.limitation
        tabs.limitSize =args.limitSize
        tabs.limitTime =args.limitTime
        tabs.disk_policy =args.disk_policy

        local r,Data,err = proxy.SetRecordOptions(tabs)

        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "设置录像参数失败！错误:"..err )
            MSG.ERROR("setRecordOptError")
        else
            MSG.OK("")
        end
    end
    return true
end

local function API_remove(red,args)
    if not args.ids or not args.channelId or not  args.diskId then
        MSG.ERROR( "argumentError" )
        return true
    end
    local ids = {}
    args.ids = cjson.decode(args.ids)
    for k,v in pairs(args.ids) do
        table.insert(ids,v)
    end
    ----args.ids---{"xx.mp4","bb.mp4"}
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.RemoveRecordFiles(args.channelId, args.diskId, ids)
        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "删除录像文件失败！错误:"..err )
            MSG.ERROR("delRecordFileError")
        else
            MSG.OK("")
        end
    end
    return true
end


--------TODO:
local function API_one_remove(red,args)
    if not args.id or not args.channelId or not  args.diskId then
        MSG.ERROR( "argumentError" )
        return true
    end
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
        MSG.ERROR( "systemError" )
    else
        local r,Data,err = proxy.RemoveRecordFiles(args.channelId, args.diskId, args.id)
        proxy:Destroy()
        if not r or not Data then
            -- MSG.ERROR( "删除录像文件失败！错误:"..err )
            MSG.ERROR("delRecordFileError")
        else
            MSG.OK("")
        end
    end
    return true
end

---------TODO:
local function API_control(red,args)
    if not args.work then
        MSG.ERROR( "无效的请求参数！" )
        return true
    end
    MSG.OK("")
    return true
end

local function API_modifyDisk(red,args)
    --初始化磁盘
    if not args.id then
        MSG.ERROR( "argumentError" )
    else
        local proxy = CO.GetProxy( "/diskmanage", "codec" )
        if not proxy then
            MSG.ERROR( "systemError" )
        else
                local r,Data,err = proxy.InitDisk(args.id)

                proxy:Destroy()
                if not r or not Data then
                    -- MSG.ERROR( "初始化磁盘失败！错误:"..err )
                    MSG.ERROR("initDiskError")
                else
                    MSG.OK("")
                    local fileProxy = CO.GetProxy("/streams", "codec")
                    if fileProxy then
                        fileProxy.DeleteFileStreamFileList("", "*")
                        fileProxy:Destroy()
                    end
                end
        end
    
    end
    return true
end

--查询磁盘信息
local function API_getdisk(red,args)
    -- NOTE: D300是读取录像文件目录信息
    if FEATURES.DEVICE_TYPE == "D300" then
        local proxy = CO.GetProxy( "/record", "codec" )
        if not proxy then
            MSG.ERROR( "systemError" )
        else
            local r, Data, err = proxy.GetSysFileDocument()
            proxy:Destroy()
            if not r or not Data then
                MSG.ERROR("getSysFileDocument err")
            else
                MSG.OK("",{
                    data = Data,
                    code = 0,
                    count = 1000,
                })
            end
        end
    else
        -- NOTE: YIDUO是读取磁盘信息
        local proxy = CO.GetProxy( "/diskmanage", "codec", "local", 20 ) --NOTE 读取需要延迟时间，否则读不到数据，原因是文件比较多需要耗费时间
        if not proxy then
            MSG.ERROR( "systemError1" )
        else
            local r,Data,err = proxy.get_disk_info()
            proxy:Destroy()
            if not r or not Data then
                -- MSG.ERROR( "查询磁盘信息失败！错误:"..err )
                MSG.ERROR("getDiskError")
            else
                ---新增的NOTE：直接把get接口和getdisk接口合并
                local result = {
                }
                local proxy_stream = CO.GetProxy( "/streams", "codec" ,"local", 20 ) --NOTE 读取需要延迟时间，否则读不到数据，原因是文件比较多需要耗费时间
                if not proxy_stream then
                    MSG.ERROR( "systemError2" )
                    return true
                else
                    local r, streams = proxy_stream.GetStreams({"local","output"})
                    proxy_stream:Destroy()

                    if not r or not streams then
                        MSG.ERROR( "getStreamsError" )
                        return true
                    end

                    local i, item
                    for i=1, #streams do
                        if not args or not args.id or args.id == streams[i].id then
                            ---TODO 判断类型为local
                            if not streams[i].sourceType or streams[i].sourceType=="main" then
                                item = {
                                    id = streams[i].id,
                                    name = streams[i].name,
                                    type = streams[i].type,
                                    sourceType = streams[i].sourceType
                                }

                                ---文件状态
                                local file_proxy = CO.GetProxy( "/record", "codec" ,"local", 20 )--NOTE 读取需要延迟时间，否则读不到数据，原因是文件比较多需要耗费时间
                                if not file_proxy then
                                    MSG.ERROR( "systemError3" )
                                    return true
                                else
                                    local r,Data1,err = file_proxy.GetChannelRecordStatus(streams[i].id)
                                    file_proxy:Destroy()
                                    if r and Data1 then
                                        if not Data1.record_info then
                                            Data1.record_info={}
                                        end
                                        item.startTime = Data1.record_info.startTime or ""
                                        item.work = Data1.work or ""
                                        item.stopTime = Data1.record_info.stopTime or ""
                                        item.totalSize = Data1.record_info.totalSize or ""
                                        item.totalDuration = Data1.record_info.totalDuration or ""
                                    end
                                end
                            end
                            table.insert( result, item )
                        end
                    end    
                    
                    if Data[1].mount or Data[1].name then 
                        Data[1].children=result 
                        -- ----之前YIDUO的实现方式
                        Data[1].id=1
                    else 
                        Data[1] = nil
                    end 

                    if Data[2].mount or Data[2].name then 
                        Data[2].children=result
                        -- ----之前YIDUO的实现方式
                        Data[2].id=2
                    else 
                        Data[2] = nil
                    end
                   
                    MSG.OK("",{
                        data=Data,
                        code = 0,
                        count = 1000,
                    })
                    return true
                end
                
                MSG.OK("",{
                    data=Data,
                    code = 0,
                    count = 1000,
                })
                return true
            end
        end
    end

    return true
end


local function API_pop(red,args)
    --弹出磁盘
    if not args.id then
        MSG.ERROR( "argumentError" )
    else
        local proxy = CO.GetProxy( "/diskmanage", "codec" )
        if not proxy then
            MSG.ERROR( "systemError" )
        else
            local r,Data,err = proxy.remove_disk(tonumber(args.id))
            proxy:Destroy()
            if not r or not Data then
                -- MSG.ERROR( "弹出磁盘失败！错误:"..err  )
                MSG.ERROR("removeDiskError")
            else
                MSG.OK("")
                local fileProxy = CO.GetProxy("/streams", "codec")
                if fileProxy then
                    fileProxy.DeleteFileStreamFileList("", "*")
                    fileProxy:Destroy()
                end
            end
        end
    
    end
    return true
end


--获取录像状态
local function API_get_record_status(red,args)
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
		MSG.ERROR( "systemError" )
    else
            local r,Data,err = proxy.get_record_status()
            proxy:Destroy()
            if not r or not Data then
                -- MSG.ERROR( "查询录像状态失败！错误:"..err )
                MSG.ERROR("getRecordStatError")
            else
                MSG.OK("",{
                    data=Data,
                    code = 0,
                    count = 1000,
                })
            end
    end

    return true
end

--转换ts--- m3u8文件
local function API_fileTranslation(red,args)
    -- local proxy = CO.GetProxy( "/record", "codec" )
    -- if not proxy then
	-- 	MSG.ERROR( "无法连接系统服务！" )
    -- else
    --         local r,Data,err = proxy.get_record_status()
    --         proxy:Destroy()
    --         if not r or not Data then
    --             MSG.ERROR( "查询录像状态失败！错误:"..err )
    --         else
                MSG.OK("","---ok")
    --         end
    -- end

    return true
end


----------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["one_remove"] = API_one_remove
dset.APIS["remove"] = API_remove
dset.APIS["record_file"] = API_record_file

dset.APIS["startAll"] = API_startAll
dset.APIS["stopAll"] = API_stopAll
dset.APIS["get_config"] = API_getconf
dset.APIS["set_config"] = API_setconf
dset.APIS["control"] = API_control
dset.APIS["get_record_status"] = API_get_record_status

dset.APIS["ejectDisk"] = API_pop
dset.APIS["getDisk"] = API_getdisk
dset.APIS["modifyDisk"] = API_modifyDisk

dset.APIS["start"] = API_start
dset.APIS["stop"] = API_stop
dset.APIS["fileTranslation"] = API_fileTranslation

--------------------------------------
return dset
