local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"

local dset = {}

local function check_usb(id)
	local proxy = CO.GetProxy( "/", "systemctrl", nil, 60 )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		local r,done,disks = proxy.EnumDisks()
		if not r or not done or not disks then
			-- MSG.ERROR( "无法枚举存储器列表！" .. (done and "TRUE" or "FALSE") .. (disks and " OK" or " FAIL" ) )
			MSG.ERROR("enumDisksError")
		else
			local i, item
			local dev
			local usbType, usbPort
			for i,item in ipairs(disks) do
				if item.devPath then
					usbType, usbPort = item.devPath:match("/hiusb%-(%w*)%..-/usb.-/%d*%-(%d*)/" )
					if usbType and tonumber(usbPort) == id then
						dev = item.device
						break
					end
				end
			end
			
			if dev then
				local speed
				if usbType == "xhci" then
					usbType = "USB3.0"
				elseif usbType == "ehci" then
					usbType = "USB2.0"
				else
					usbType = "USB1.x"
				end
				r,done,speed = proxy.TestDiskRead( dev, 4096, 10240 )
				if not r or not done then
					MSG.OK( "发现存储设备(" .. usbType .. ")，但读取失败" )
				else
					MSG.OK( "发现存储设备(" .. usbType .. "), 读取速度：" .. (speed or "0") .. "/s" )
				end
			else
				MSG.ERROR( "notFindDiskError" )
			end
		end
		proxy:Destroy()
	end

	return true
end

local function check_sata(id)
	local proxy = CO.GetProxy( "/", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		local r,done,disks = proxy.EnumDisks()
		if not r or not done or not disks then
			MSG.ERROR( "enumDisksError" )
		else
			local i, item
			local dev
			for i,item in ipairs(disks) do
				if item.devPath then
					local mth = item.devPath:match("/ata(%d+)/" )					
					if mth and tonumber(mth) == id then
						dev = item.device
						break
					end
				end
			end
			
			if dev then
				local speed
				r,done,speed = proxy.TestDiskRead( dev, 4096, 10240 )
				if not r or not done then
					MSG.OK( "发现硬盘，但读取失败" )
				else
					MSG.OK( "发现硬盘, 读取速度：" .. (speed or "0") .. "/s" )
				end
			else
				MSG.ERROR( "notFindDiskError" )
			end
		end
		proxy:Destroy()
	end

	return true
end

local function check_serial(flag)
	local proxy = CO.GetProxy( "/serial", "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		if flag == 1 then
			proxy.Testing(false)
		else
			proxy.Testing(true)
		end
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function check_buzzer(flag)
	local proxy = CO.GetProxy( "/", "ui" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		if flag == 1 then
			proxy.ForceLED( "beep", "OFF", 4 )
		else
			proxy.ForceLED( "beep", "ON", 40 )
		end
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function check_alarm(flag)
	local proxy = CO.GetProxy( "/", "ui" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		if flag == 1 then
			proxy.Testing(false)
		else
			proxy.Testing(true)
		end
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function check_resetSwitch(flag)
	local proxy = CO.GetProxy( "/", "ui" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		if flag == 1 then
			proxy.Testing(false)
		else
			proxy.Testing(true)
		end
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function check_sataled(sataid,color)
	local proxy = CO.GetProxy( "/diskmanage", "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		proxy.change_disk_status(sataid,color)
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function API_exec(red, args)
	if args and args.id and args.name then
		if args.name == "usb1" then
			return check_usb(1)
		elseif args.name == "usb2" then
			return check_usb(2)
		elseif args.name == "sata1" then
			return check_sata(1)
		elseif args.name == "sata2" then
			return check_sata(2)
		elseif args.name == "serial" then
			return check_serial( tonumber(args.status) or 0 )
		elseif args.name == "buzzer" then
			return check_buzzer( tonumber(args.status) or 0 )
		elseif args.name == "alarm" then
			return check_alarm( tonumber(args.status) or 0 )
		elseif args.name == "resetSwitch" then
			return check_resetSwitch( tonumber(args.status) or 0 )
		elseif args.name == "ledtest" and args.sataid and args.color then
			return check_sataled( tonumber(args.sataid),tonumber(args.color) )
		else
			MSG.ERROR( "invalidArgError" )
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

dset.APIS = dset.APIS or {}

dset.APIS["exec"] = API_exec

return dset

