local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"

local dset = {}
--------------------------------------
local function API_get(red, args)
	local proxy = CO.GetProxy( "/config/codec", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	end

	local configs = proxy._P( "mainStream" )
	proxy:Destroy()

	if not configs then
		MSG.ERROR( "getVideoArgError" )
		return true
	end

	local retData={}

	local i
	for i=1, #configs do
		if not args or not args.id or tonumber(args.id) == i then
			local item = {}
			item.name = "暂不支持"
			item.audioSrc = configs[i].audio.source
			item.scale_mode = configs[i].video.scale_mode
			item.grey = configs[i].video.grey
			item.profile = configs[i].video.profile
			item.brCtrl = configs[i].video.brCtrl
			item.bitrate = configs[i].video.bitrate
			item.framerate = configs[i].video.framerate
			item.gop = configs[i].video.gop
			item.refCount = configs[i].video.refCount
			table.insert( retData, item )
		end
	end

	MSG.OK( "", {
	data = retData
	})
	return true
end

--------------------------------------
local function API_sub_get(red, args)
	local proxy = CO.GetProxy( "/config/codec", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	end

	local configs = proxy._P( "subStream" )
	proxy:Destroy()

	if not configs then
		MSG.ERROR( "getVideoArgError" )
		return true
	end

	local retData={}

	local i
	for i=1, #configs do
		if not args or not args.id or tonumber(args.id) == i then
			local item = {}
			item.enable = 1 --TODO
			item.name = "暂不支持"
			item.audioSrc = configs[i].audio.source
			item.scale_mode = configs[i].video.scale_mode
			item.grey = configs[i].video.grey
			item.profile = configs[i].video.profile
			item.brCtrl = configs[i].video.brCtrl
			item.bitrate = configs[i].video.bitrate
			item.framerate = configs[i].video.framerate
			item.gop = configs[i].video.gop
			item.refCount = configs[i].video.refCount
			table.insert( retData, item )
		end
	end

	MSG.OK( "", {
	data = retData
	})
	return true
end

--------------------------------------

local function API_modify(red, args)
	if args and args.id and tonumber(args.id) and tonumber(args.id) > 0 and tonumber(args.id) <= 2 then
		local proxy = CO.GetProxy( "/config/codec", "codec" )
		if not proxy then
			MSG.ERROR( "codecError" )
			return true
		end

		local sets={}
		sets[ "mainStream[" .. args.id .. "].audio.source" ] = args.audioSrc
		sets[ "mainStream[" .. args.id .. "].video.scale_mode" ] = args.scale_mode
		sets[ "mainStream[" .. args.id .. "].video.grey" ] = tonumber(args.grey)
		sets[ "mainStream[" .. args.id .. "].video.profile" ] = args.profile
		sets[ "mainStream[" .. args.id .. "].video.brCtrl" ] = args.brCtrl
		sets[ "mainStream[" .. args.id .. "].video.bitrate" ] = tonumber(args.bitrate)
		sets[ "mainStream[" .. args.id .. "].video.framerate" ] = tonumber(args.framerate)
		sets[ "mainStream[" .. args.id .. "].video.gop" ] = tonumber(args.gop)
		sets[ "mainStream[" .. args.id .. "].video.refCount" ] = tonumber(args.refCount)

		local r, rr = proxy:P_SET(sets) 
		proxy:Destroy()
		if r and rr then
			MSG.OK("")
		else
			MSG.ERROR( "setCodecArgError" )
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

--------------------------------------
local function API_sub_modify(red, args)
	if args and args.id and tonumber(args.id) and tonumber(args.id) > 0 and tonumber(args.id) <= 2 then
		local proxy = CO.GetProxy( "/config/codec", "codec" )
		if not proxy then
			MSG.ERROR( "无法连接编码服务" )
			return true
		end

		local sets={}
		sets[ "subStream[" .. args.id .. "].audio.source" ] = args.audioSrc
		sets[ "subStream[" .. args.id .. "].video.scale_mode" ] = args.scale_mode
		sets[ "subStream[" .. args.id .. "].video.grey" ] = tonumber(args.grey)
		sets[ "subStream[" .. args.id .. "].video.profile" ] = args.profile
		sets[ "subStream[" .. args.id .. "].video.brCtrl" ] = args.brCtrl
		sets[ "subStream[" .. args.id .. "].video.bitrate" ] = tonumber(args.bitrate)
		sets[ "subStream[" .. args.id .. "].video.framerate" ] = tonumber(args.framerate)
		sets[ "subStream[" .. args.id .. "].video.gop" ] = tonumber(args.gop)
		sets[ "subStream[" .. args.id .. "].video.refCount" ] = tonumber(args.refCount)

		local r, rr = proxy:P_SET(sets) 
		proxy:Destroy()
		if r and rr then
			MSG.OK("")
		else
			MSG.ERROR( "提交参数到编码服务失败" )
		end
	else
		MSG.ERROR( "无效的提交参数" )
	end
	return true
end

--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["modify"] = API_modify
dset.APIS["get_sub"] = API_sub_get
dset.APIS["modify_sub"] = API_sub_modify

--------------------------------------
return dset

