#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

local _done, features = pcall(require, "features")
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"

--NOTE by左振宇: 注意这里！这时导致快速复位被憋死的主要原因！
--任何看到这里人请注意：请自行学习关于socket或文件fd的close-on-exec的基本概念。
--目前，由于执行了ntfs挂载等后台服务，都是由这个进程 fork 出来并 exec 的。
--如果我们打开的套接字没有标注为 close-on-exec，那么这些套接字将被fork出的后
--台进程继承（最大的可能是ntfs-3g）。那么，我们连接到 coserver 的proxy连接也会被 
--ntfs-3g程序继承。因此，即使 codec 程序被杀死了，这个proxy连接也不会断开，所以
--会导致coserver一直认为codec服务在线，并且将CoAF请求消息转发到这个“已经死了”
--的套接字上，所以出现服务不可访问的错误！

socket.default_close_on_exec = true

local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local event = require "coaf.event"

local global=require "soc.global"
local capture=require "soc.capture"
local decoder=require "soc.decoder"
local encoder=require "soc.encoder"
local output=require "soc.output"
local ain = require "soc.ain"

local mo_Encoder = require "mod_Encoder"
local mo_Decoder = require "mod_Decoder"
local mo_GeneralStream = require "mod_GeneralStream"
local mo_NetStream = require "mod_NetStream"

local mo_Sip = nil
if features.FEATURE_SIP or features.FEATURE_GB28181 then
	mo_Sip = require "mod_Sip"
end

local mo_Output = require "mod_Output"
local mo_Streams = require "mod_Streams"
local mo_Streamer = require "mod_Streamer"
local mo_AudioIn = require "mod_AudioIn"
local mo_AudioOut = require "mod_AudioOut"
local mo_AudioEncoder = require "mod_AudioEncoder"
local mo_fpgaController = require "mod_fpgaController"
local mo_Discovery = require "mod_Discovery"
local mo_Preview = require "mod_Preview"

local mo_Devices = nil
local mo_DiskManage = nil
local mo_Record = nil
local mo_Record_setting = nil
local cfg_record = nil

if features.FEATURE_LOCAL_STREAM then
	mo_Devices = require "mod_Devices"
end

if features.FEATURE_RECORD then
	mo_DiskManage = require "mod_diskManage"
	mo_Record = require "mod_record"
	mo_Record_setting = require "configs.record_setting"
	cfg_record = mo_Record_setting.Load()
else 
	-----add hyl
	mo_Record = require "mod_record"
end



local mo_Layout = require "mod_Layout"

local sys_config = require "sys_config"
-- add by kiloview@2020-9-28 use D300??
local sys_audio_config = require "sys_audio_config"
local sys_video_config = require "sys_video_config"

local mo_Webrtc = require "mod_Webrtc"



local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/codec.log")
	if not logF or not logF:init() then
		_dbg.Error("CODEC", "Fail to open log file: /var/log/codec.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "codec_service" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "CODEC" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

_G.g_event = event.createNew( "codec" )


-- ** CoService **
local cosvc_Codec
------------------

-- **200ms period checker [normal check]**
--  E-DOCTOR add ndi decode
local flag_output_updating, flag_ndi_discovering

--NOTE: Since something update require long time and make thread yield, I **MUST**
--      flag it is working or not, to avoid re-entry.
local function for200ms_PeriodCheck()
	if not flag_output_updating then
		flag_output_updating = true
		mo_Output.Update()			--Update output (that's cleanup noused streams)
		mo_Preview.Update()
		flag_output_updating = false
	end
end

local function for15sTimer_NDICheck()
	if not flag_ndi_discovering then
		flag_ndi_discovering = true
		mo_Discovery.Step()
		flag_ndi_discovering = nil
	end
end

local flag_devices_updating
local function for3s_PeriodCheck()
	if not flag_devices_updating then
		flag_devices_updating = true
		mo_Devices.Refresh()		--Refresh device online status and create streams.
		flag_devices_updating = false
	end
end
-- **50ms period checker [fast check]**
-- To check: [?OSD?]
local function for50ms_PeriodCheck()
end

-- ** Timers **
local for200msTimer
local for50msTimer
local for3sTimer
local for15sTimer

local hdOut0, hdOut1, sdOut, preview

-- **Main entry**
local function Main(args)
	cosvc_Codec = service.New( "codec", "unix:/var/run/codec.sock" )
	if cosvc_Codec then
		_dbg.Info( "CODEC", "CoService of 'codec' is created." )
		cosvc_Codec:CreateCoServerProxy()
	else
		_dbg.Error( "CODEC", "Fail created CoService!" )
	end

	-- Init global HiMPP
	global.Init()

	-- Init each modules.
	capture.Init()
	encoder.Init()
	output.Init()
	decoder.Init()

	--NOTE here: the audio-in module 'RESET' the analog codec.
	ain.Init(true)
	
	--Init FPGA controller before other modules.
	mo_fpgaController.Init()

	--Create and start audio capture channel.
	mo_AudioIn.Init() -- todo add by kiloview@2020-9-28 mo_AudioIn.Init( {gain=0} )
	mo_AudioOut.Init()

	mo_Decoder.Init()
	mo_GeneralStream.Init( g_event, "codec" )

	mo_NetStream.Init()

	if mo_Sip then
		mo_Sip.Init()
	end
	mo_Layout.Init()

	mo_Streams.Init()
	mo_Output.Init()
	-- mo_Encoder.Init()
	
	if mo_DiskManage and mo_Record then
		mo_DiskManage.Init(g_event, "codec")
		mo_Record.Init(cfg_record, g_event, "codec")
	else 
		mo_Record.Init(cfg_record, g_event, "codec")
	end

	hdOut0 = mo_Output.CreateOutput( "hd0", "hd:0", 1 )
	hdOut1 = mo_Output.CreateOutput( "hd1", "hd:1", 2 )
	
	--TODO: Initialize SD output.
	--sdOut = mo_Output.CreateOutput( "sd", "sd:0" )
	
	if hdOut0 then
		hdOut0.Init(sys_video_config.outputs.hd0_disabled_interfaces)
	end

	if hdOut1 then
		hdOut1.Init(sys_video_config.outputs.hd1_disabled_interfaces)
	end

	--if sdOut then
	--	sdOut.Init(sys_video_config.sd0_disabled_interfaces)
	--end
	
	if mo_Devices then
		mo_Devices.Init()
	end
	mo_Streamer.Init(cosvc_Codec)

	mo_Webrtc.Init()

	--for50msTimer = coaf.NewTimer( for50ms_PeriodCheck, 0.01, 0.05 )
	--if for50msTimer then
	--	for50msTimer:start()
	--end

	if hdOut0 then
		hdOut0.Start()
	end

	if hdOut1 then
		hdOut1.Start()
	end

	--if sdOut then
	--	sdOut.Start()
	--end

	--NOTE: For Hi3535 (or others), BT1120 with some bugs.
	if sys_video_config.patch_bt1120_bug then
		if hdOut0 then
			hdOut0.PatchBT1120Bug()
		end
		if hdOut1 then
			hdOut1.PatchBT1120Bug()
		end
	end
	
	if mo_Sip then
		mo_Sip.CreateInstance()
		mo_Sip.SetupAccount()
	end

	mo_AudioEncoder.Init()
	--创建(本地和Output)编码模块: 待Output模块初始化后再处理编码

	mo_Encoder.Init()
	mo_Encoder.CreateStreams()

	if cosvc_Codec then
		cosvc_Codec:ExposeObject("/streams", mo_Streams )
		cosvc_Codec:ExposeObject("/streamer", mo_Streamer )
		cosvc_Codec:ExposeObject("/webrtc", mo_Webrtc )
		cosvc_Codec:ExposeObject("/layout", mo_Layout )

		cosvc_Codec:ExposeObject("/ain", mo_AudioIn )
		cosvc_Codec:ExposeObject("/output/hd0", hdOut0 )
		cosvc_Codec:ExposeObject("/output/hd1", hdOut1 )

		mo_Preview.CreateOutput( "preview", "preview" )
		cosvc_Codec:ExposeObject("/preview", mo_Preview)
		--cosvc_Codec:ExposeObject("/output/sd", sdOut )
		--For compatible with old API, test only.
		cosvc_Codec:ExposeObject("/output", hdOut0 )
		cosvc_Codec:ExposeObject("/mo_output", mo_Output )
		if mo_Sip then
			cosvc_Codec:ExposeObject("/sip", mo_Sip )
		end
		
		if mo_DiskManage and mo_Record then 
			cosvc_Codec:ExposeObject("/diskmanage", mo_DiskManage)
			cosvc_Codec:ExposeObject("/record", mo_Record)
		else
			cosvc_Codec:ExposeObject("/record", mo_Record) ------ add hyl
		end
		
		if mo_Devices then
			cosvc_Codec:ExposeObject("/device", mo_Devices )
		end

		cosvc_Codec:ExposeObject("/encoder", mo_Encoder )
		cosvc_Codec:ExposeObject("/audioEncoder", mo_AudioEncoder)
		--Discovery service.
		--It will expose "/discovery" and "/discovery/ndi"
		--In future, it may also expose other discovery types, such as "/discovery/onvif", ... etc.
		mo_Discovery.Init(cosvc_Codec)
	end

	for200msTimer = thread.NewTimer( for200ms_PeriodCheck, 0.01, 0.2 )
	if for200msTimer then
		for200msTimer:start()
	end

	for15sTimer = thread.NewTimer( for15sTimer_NDICheck, 5, 15)
	if for15sTimer then
		for15sTimer:start()
	end
	
	if mo_Devices then
		for3sTimer = thread.NewTimer( for3s_PeriodCheck, 0.01, 3 )
	end
	
	if for3sTimer then
		for3sTimer:start()
	end

end

coaf.Start(Main, arg)

if for3sTimer then
	for3sTimer:stop()
end


if for200msTimer then
	for200msTimer:stop()
end

if for15sTimer then
	for15sTimer:stop()
end

if for50msTimer then
	for50msTimer:stop()
end

event.free( "codec" )

ain.Cleanup()
output.Cleanup()
decoder.Cleanup()
encoder.Cleanup()
capture.Cleanup()
global.Cleanup()

_dbg.Print( "CODEC", "Service terminated." )

