local sys_config = require "sys_config"
local ain = require "soc.ain"
local ain_cfg = require "configs.ain_config"

--TODO: Fix this as a reasonable value.
local MIC_BASE_GAIN = 9
local LINE_BASE_GAIN = 6

local _M = {
	source = "LINEIN",
	ainChannel = nil,
	config = nil
}

local function CalcGain(g)
	local baseGain = LINE_BASE_GAIN
	if _M.source == "MIC" or _M.source == "MICIN" then
		baseGain = MIC_BASE_GAIN
	end

	g = math.ceil((g + baseGain*2)/2)
	if g < 0 then
		return 0
	elseif g > 15 then
		return 15
	else
		return g
	end
end

function _M.SelectSource(src)
	_M.source = src or "LINEIN"
	if _M.source ~= "LINEIN" and _M.source ~= "LINE" then
		_M.source = "MIC"
	end

	_M.config.source = _M.source
	_M.config:__TRIGGER(true)

	if _M.ainChannel then
		_M.ainChannel:SelectSource( _M.source )
	end
end

function _M.Init()
	if not _M.config then
		_M.config = ain_cfg.Load()
		_M.source = _M.config.source
	end

	if not _M.ainChannel then
		_M.ainChannel = ain.CreateChannel(1)
		_M.ainChannel:Init()
		_M.ainChannel:Start()
		_M.ainChannel:SelectSource( _M.source )
		_M.ainChannel:SetGain( CalcGain( _M.config.gain or 0 ) )
	end
end

function _M.UpdateConfig(cfg)
	if cfg and _M.config then
		_M.config.source = cfg.source or _M.config.source
		_M.config.gain = tonumber(cfg.gain) or _M.config.gain
		_M.config:__TRIGGER(true)
		_M.source = _M.config.source
	end

	if cfg and _M.ainChannel then
		_M.ainChannel:SelectSource( _M.source )
		_M.ainChannel:SetGain( CalcGain( _M.config.gain or 0 ) )
	end
end

function _M.GetConfig()
	return _M.config
end

return _M

