local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local aout = require "soc.aout"
local sys_audio_config = require "sys_audio_config"
local hiperiphery = require "hiperiphery"
local HIMM = hiperiphery.HIMM
local _done, features = pcall(require, "features")
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end
local pretty = require "pl.pretty"
require "kmCore"

local _M = {
	--混音器
	--混音器的数量根据 outputs 配置文件来创建
	mixer = sys_audio_config.outputs,
	--解码音频的派发器
	--派发器根据业务逻辑派发到 sink，在把 sink 挂载到 mixer
	dispatcher = {},
	--
	mSink ={}
}

function EnableLineOut(flag)
	if (not flag) then
		flag = true
	end

	local ttl=0x40	
	if (flag == false) then
		ttl=0x00		
	end

	-- 3536 GPIO-15_6 
	HIMM.write( 0x120f01e4, 0x1 )
	HIMM.write( 0x12240400, 0x40 )
	HIMM.write( 0x12240100, ttl )	
end

function _M.Init()
	aout.Init() --NOT reset the codec

	EnableLineOut()

	local k,v
	local args
	for k,v in pairs(_M.mixer) do
		--海思硬件相关
		--print("#####Init ", k, v.sink_type, v.audio_type, v.sampling, v.channels, v.sampleSize)
		if v.sink_type == "HISI" or v.sink_type == "HISIAO" then
			v.aoChannel = aout.CreateChannel( v.aoID, v.audio_type == "analog" ) --Connected DAC?
			if not v.aoChannel then
				_dbg.Error( "AUDIO-OUT", "Fail to create/initialize audio output channel:", v.aoID, "of Hi-AO, for:", k )
			else
				v.aoChannel:SetAudioOption({
					sampling = v.sampling,
					channels = v.channels,
					sampleSize = v.sampleSize
				})
				v.aoChannel:Init()
				v.aoChannel:SetGain( v.volume )
				if v.analog_gain then
					v.aoChannel:SetAnalogGain( v.analog_gain )
				end
				if v.mute then
					v.aoChannel:Mute()
				else
					v.aoChannel:Unmute()
				end
			end
		end
		
		--创建播放音频终端sink
		local outSink = KM_STREAMING.AudioEngine.CreateAudioSink( v.sink_type, v.sink_options or v.device )
		if outSink then
			--创建音频混音器，一个播放终端一个混音器
			local audioMixer = KM_STREAMING.AudioEngine.CreateAudioMixing( {name="mixer"..k, sampling=v.sampling, channels=v.channels, sampleSize=v.sampleSize} )
			-- print("KM_STREAMING.AudioEngine.CreateAudioMixing. ", "mixer"..k)
			if audioMixer then
				audioMixer:AddOutputSink (outSink)
				audioMixer:Set( v.sampling, v.channels, v.sampleSize, 16 ) --Default 16bits width.
				audioMixer:Open()
				_M.mixer[k].outSink    = outSink     --
				_M.mixer[k].audioMixer = audioMixer  --
			end
		end
	end
	-- _M.mixer:Start()
	--pretty.dump(_M.mixer)
end

function _M.AddOutputSink( outName, id, sink_type, sink_options )
	
end

function _M.RemoveOutputSink( outName, id )
end

function _M.GetOutputSink( output_name )
	if output_name then
		local o = _M.mixer[output_name]
		if o and o.outSink then
			return o.outSink
		end
	end
	return nil
end

function _M.GetOutputs()
	
end
--设置硬件输出的声音
--HDMI LINEOUT VS0
--gain or "mute" or "unmute"
function _M.SetOutputVolume( output_name, gain )
	if output_name then
		local o = _M.mixer[output_name]
		if not o then
			_dbg.Error( "AUDIO-OUT", "Invalid output name:", output_name, "while setting output volume!" )
			return
		end
		-- print("######---------SetOutputVolume------------", output_name, gain)
		--HI3535采用软件控制增益
		if features.SOFTWARE_OUTPUT_VOLUME then
			if not o.outSink then
				_dbg.Error("outSink", "Invalid output name:", output_name, "while setting output volume!" )
				return
			end
			if gain == "mute" then
				o.mute = true
				o.outSink:Mute(true)
			elseif gain == "unmute" then
				o.mute = false
				o.outSink:Mute(false)
			else
				gain =tonumber(gain)
				if gain then
					gain = math.floor(gain)
					if gain < -121 then
						gain = -121
					elseif gain > 6 then
						gain = 6
					end
					--将DB值转换为val
					local gain_val = math.floor((gain + 121) / (121 + 6) * 200)
					o.outSink:Mute(false)
					o.outSink:SetGainRange(200, -51, 20)
					if o.audio_type == "analog" then
						--gain = 6 - gain
						o.analog_gain = gain
						o.outSink:SetGainValue(gain_val, gain_val)
					else
						o.volume = gain
						o.outSink:SetGainValue(gain_val, gain_val)
					end
				end
			end
		else
			if o.sink_type == "HISI" or o.sink_type == "HISIAO" then
				if not o.aoChannel then
					_dbg.Error( "AUDIO-OUT", "Audio output name:", output_name, " is not initalized while setting output volume!" )
					return
				end

				if gain == "mute" then
					o.mute = true
					o.aoChannel:Mute()
				elseif gain == "unmute" then
					o.mute = false
					o.aoChannel:Unmute()
				else
					gain =tonumber(gain)
					if gain then
						gain = math.floor(gain)
						if gain < -121 then
							gain = -121
						elseif gain > 6 then
							gain = 6
						end

						if o.audio_type == "analog" then
							gain = 6 - gain
							o.analog_gain = gain
							o.aoChannel:SetAnalogGain(gain)
						else
							o.volume = gain
							o.aoChannel:SetGain(gain)
						end
					end
				end
			elseif o.sink_type == "ALSA" then --TODO: Alsa audio gain set
			end
		end
	end
end

function _M.GetOutputVolume( output_name )
	if output_name then
		local o = _M.mixer[output_name]
		if not o then
			_dbg.Error( "AUDIO-OUT", "Invalid output name:", output_name, "while getting output volume!" )
			return nil
		end
		
		if o.sink_type == "HISI" or o.sink_type == "HISIAO" then
			if not o.aoChannel then
				_dbg.Error( "AUDIO-OUT", "Audio output name:", output_name, " is not initalized while getting output volume!" )
				return nil
			end

			if o.mute then
				return "mute"
			else
				if o.audio_type == "analog" then
					return 6 - o.analog_gain
				else
					return o.volume
				end
			end
		elseif o.sink_type == "ALSA" then
			--TODO: Get alsa audio gain.
			return nil
		end
	end
end

function _M.SetOutputFormat( output_name, opts )
	if not _M.mixer then
		_dbg.Error( "AUDIO-OUT", "The default mixer is not initialized while set output format!" )
		return false
	end
	-- print("###### SetOutputFormat:", output_name)
	if output_name then
		local o = _M.mixer[output_name]
		if not o then
			_dbg.Error( "AUDIO-OUT", "Invalid output name:", output_name, "while setting output format!" )
			return false
		end
		
		--[[
		if not o.aoChannel then
			_dbg.Error( "AUDIO-OUT", "Audio output name:", output_name, " is not initalized while setting output format!" )
			return false
		end
		]]

		o.sampling = opts.sampling or o.sampling
		o.channels = opts.channels or o.channels
		o.sampleSize = opts.sampleSize or o.sampleSize

		-- local tap = _M.mixer:GetMixOutputTap( output_name )
		-- if not tap then
		-- 	_dbg.Error( "AUDIO-OUT", "Audio output name:", output_name, " is not initialized by mixer!" )
		-- 	return false
		-- end

		-- local disp = tap:GetAudioDispatcher()
		-- if not disp then
		-- 	_dbg.Error( "AUDIO-OUT", "Audio output name:", output_name, " audio dispatcher is not initialized by mixer!" )
		-- 	return false
		-- end

		-- disp:Set( o.sampling, o.channels, o.sampleSize, 16 ) --Default is 16bits width.

		if (o.sink_type == "HISI" or o.sink_type == "HISIAO") and o.aoChannel then
			o.aoChannel:SetAudioOption({
				sampling = o.sampling,
				channels = o.channels,
				sampleSize = o.sampleSize
			})
		elseif o.sink_type == "ALSA" then
			--TODO: ALSA audio format setting.
		end

		return true
	else
		return false
	end
end

function _M.GetOutputEncoder( output_name, codecArgs )
	--TODO:
end

function _M.FreeOutputEncoder( output_name, encObj )
	--TODO:
end

--对每一路解码出来的音频创建派发器
function _M.GetChannelSink( channel )
	-- print("###### GetChannelSink:", channel)
	local name = "C-" .. channel
	if _M.dispatcher[name] then
		return _M.dispatcher[name]
	end
	--创建派发器
	_M.dispatcher[name] = KM_STREAMING.AudioEngine.CreateAudioDispatcher( name )
	-- _M.dispatcher[name]:Set( v.sampling, v.channels, v.sampleSize, 16 )
	-- _M.dispatcher[name]:Set( 48000, 2, 1024, 16 )
	_M.dispatcher[name]:Start()
	return _M.dispatcher[name]
end

function _M.FreeChannelSink( channel )
	local name = "C-" .. channel
	if _M.dispatcher[name] then
		KM_STREAMING.AudioEngine.FreeAudioDispatcher(_M.dispatcher[name])
		_M.dispatcher[name] = nil;
	end
end

function _M.GetChannelEncoder( channel, output_name, codecArgs )
	--TODO:
end

function _M.FreeChannelEncoder( channel, output_name, encObj )
	--TODO:
end

function _M.ConnectChannelOutput( channel, output_name )
	local name  = "C-" .. channel
	local Sname = name..output_name
	if _M.dispatcher[name] then --派发器已经创建
		if _M.mixer[output_name] and _M.mixer[output_name].audioMixer then --创建了混音器
			--创建 "MIXER" sink
			local mSink = KM_STREAMING.AudioEngine.CreateAudioSink( "MIXER", {
						fixed_channels = _M.mixer[output_name].channels, 
						fixed_sampling = _M.mixer[output_name].sampling} )
			if mSink then
				--把sink挂载到派发器的输出
				_M.dispatcher[name]:AddOutputSink( mSink );
				--把sink挂载到混音器的输入
				_M.mixer[output_name].audioMixer:AddInputSource( mSink );
				_M.mSink[Sname] = mSink
			end
		end
	end
	-- print("######## ConnectChannelOutput.", Sname)
	-- pretty.dump(_M.dispatcher)
	-- pretty.dump(_M.mSink)
end

function _M.DisconnectChannelOutput( channel, output_name )
	local name  = "C-" .. channel
	local Sname = name..output_name
	if _M.mSink[Sname] then
		if _M.dispatcher[name] then
			_M.dispatcher[name]:RemoveOutputSink(_M.mSink[Sname])
		end
		if _M.mixer[output_name] and _M.mixer[output_name].audioMixer then
			_M.mixer[output_name].audioMixer:RemoteInputSource(_M.mSink[Sname])
		end
		KM_STREAMING.AudioEngine.FreeAudioSink(_M.mSink[Sname])
		_M.mSink[Sname] = nil
	end
	-- print("######## DisconnectChannelOutput.", Sname)
	-- pretty.dump(_M.dispatcher)
	-- pretty.dump(_M.mSink)
end

return _M
