local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

local _done, features = pcall(require, 'features')
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

--local posix_time = require "posix.time"
local ndi_discovery

if not features.FEATURE_DISABLE_NDI then
	ndi_discovery = require "mod_NDIDiscovery"
end

local disco_config = require "configs.discovery_config"

local _M = {
	cfg = nil
}

function _M.Init(cosvc)
	_M.cfg = disco_config.Load()

	if ndi_discovery then
		ndi_discovery.Init()
		_M.NDI_UpdateWorkDiscovery()
	end

	cosvc:ExposeObject("/discovery", _M)

	if ndi_discovery then
		cosvc:ExposeObject("/discovery/ndi", ndi_discovery)
	end
end

function _M.Step()
	if ndi_discovery then
		ndi_discovery.Step()
		--TODO: Update sources (url) if neccesary.
	end
end

function _M.NDI_GetDiscoveryManuals()
	return _M.cfg.ndi.manuals, _M.cfg.ndi.groups
end

function _M.NDI_SetDiscoveryServer(enable, addr)
	if _M.cfg.ndi then
		if _M.cfg.ndi.use_discovery_server ~= enable or (addr and _M.cfg.ndi.discovery_server ~= addr) then
			_dbg.Info("DISCOVERY", "Update/Chagned the discovery by : using = ", enable, ", address = ", addr)
			_M.cfg.ndi.use_discovery_server = enable
			if addr then
				_M.cfg.ndi.discovery_server = addr
			end
			_M.cfg:__TRIGGER(true)

			if _M.cfg.ndi.use_discovery_server then
				ndi_discovery.SetDiscoveryServer(_M.cfg.ndi.discovery_server)
			else
				ndi_discovery.SetDiscoveryServer("")
			end

			--TODO:关闭所有 NDI流 再重新打开 
			ndi_discovery.RestartNDIStream(_M.cfg.ndi)
		end
	end
	return true
end

function _M.NDI_GetDiscoveryServer()
	if _M.cfg.ndi then
		return _M.cfg.ndi.use_discovery_server, _M.cfg.ndi.discovery_server
	end
end

function _M.NDI_SetDiscoveryManuals(ips, groups)
	if type(ips) ~= "table" then
		ips = { ips }
	end
	if type(groups) ~= "table" then
		groups = { groups }
	end
	_M.cfg.ndi.manuals = ips
	_M.cfg.ndi.groups = groups
	_M.cfg:__TRIGGER(true)
	_M.NDI_UpdateWorkDiscovery()
	return true
end

function _M.NDI_UpdateWorkDiscovery()
	local groups = {["public"] = true, "public"}
	local manual_ips = {}

	--Get from manual groups.
	for i=1, #_M.cfg.ndi.groups do
		if not groups[_M.cfg.ndi.groups[i]] then
			groups[_M.cfg.ndi.groups[i]] = true
			table.insert(groups, _M.cfg.ndi.groups[i])
		end
	end

	--Get from manual list.
	for i=1, #_M.cfg.ndi.manuals do
		if not manual_ips[_M.cfg.ndi.manuals[i]] then
			manual_ips[_M.cfg.ndi.manuals[i]] = true
			table.insert(manual_ips, _M.cfg.ndi.manuals[i])
		end
	end
	
	if ndi_discovery then
		ndi_discovery.SetWorkGroups( groups )
		ndi_discovery.SetWorkManuals( manual_ips )
		if _M.cfg.ndi.use_discovery_server then
			ndi_discovery.SetDiscoveryServer(_M.cfg.ndi.discovery_server)
		else
			ndi_discovery.SetDiscoveryServer("")
		end
	end
end

--TODO: For other discovery types.

return _M
