local sys_config = require "sys_config"
local mod_Decoder = require "mod_Decoder"
local mod_GenStream = require "mod_GeneralStream"

local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

local _M = {
}

local stream_meta = {}

function stream_meta:SpecialInit()
	--Step2: Create fake media source
	self.fakeSource = KM_STREAMING.FakeSource.createNew()
	if not self.fakeSource then
		_dbg.Error( "DUMMY_STREAM", self.id, ": Fail to create fake media source" )
		self:Destroy()
		return false
	end

	self.syncSource:BindMediaSource( self.fakeSource:GetMediaSource(), true ) --I can reuse the crossbuffer
	--TODO: Start seek the network stream?
	return true
end

function stream_meta:SpecialDestroy()
	self.fakeSource = nil
end

function stream_meta:SpecialStart(group, action)
	_dbg.Info( "DUMMY_STREAM", self.id, ": Started" )
	return true
end

function stream_meta:SpecialStop(group, action)
	_dbg.Info( "DUMMY_STREAM", self.id, ": Stopped" )
	return true
end

function stream_meta:GetStreamStatus(group)
	local with_act, acts = self:CheckStartActions()
	local output_start = false
	local stream_start = false
	if with_act then
		local k,v
		for k,v in pairs(acts) do
			if k == "output" then
				output_start = true
				stream_start = true
				break
			else
				stream_start = true
			end
		end
	end

	local ret = {
		output_start = output_start,
		stream_start = stream_start
	}

	if not self.fakeSource then
		ret.live = "closed"
		ret.resolution = "?x?"
		ret.bitrate = 0
		ret.video = "NONE"
		ret.audio = "NONE"
	else
		ret.live = "on"
		ret.resolution = "DUMMY"
		ret.bitrate = 128
		ret.video = "H264"
		ret.audio = "PCM"
	end
	return ret
end

function stream_meta:GetMediaSource()
	if self.fakeSource then
		return self.fakeSource:GetMediaSource()
	else
		return nil
	end
end

stream_meta = mod_GenStream.ConstructStreamMeta( stream_meta )

----------------------------------

function _M.Init()
end

function _M.Create(id)
	local stream = mod_GenStream.CheckStreamExist(id)
	if stream then
		_dbg.Warn( "DUMMY_STREAM", "The creating stream '" .. id .. "' is already exist." )
		return stream
	end

	local buffer_option = "live:200:200:500:0:0"

	local min_group, decoder_group = mod_Decoder.AllocateDecoderGroup()
	stream = {
		id = id,
		type = "unknown",
		decoder_group_id = min_group,
		decoderGroup = decoder_group,
		decoder_channel = nil,
		start_actions = {},
		audio_outputs = {},
		audio_enabled = {},
		vpss_using = {},
		current_audio_intfs = {},
		stream_openned = false,
		buffer_option = buffer_option,
		url = nil,
		stream_options = nil
	}

	setmetatable( stream, stream_meta )
	stream:Init()
	return stream
end

return _M

