local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local streams = require "mod_Streams"
local soc_encoder = require "soc.encoder"
local capture = require "soc.capture"
local _done, features = pcall(require, "features")
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

local pt = require "pl.pretty"

local _M = {
	encoders_config = nil,
	local_out_encoders = {},
	vi_cap = nil,
	vi_cap_vpss_group = nil,
	vi_cap_vpss0 = nil,
	vi_cap_vpss1 = nil,
	wbc_obj = nil,
	outputid = 0
}

-------------------------------------------------------
-- Output (Wbc=Write-back channel) Encoding
-------------------------------------------------------

local encoder_meta = {}

function encoder_meta:Ex_Init()
	_dbg.Info( "ENCODER", "------Ex_Init self.encoder_id ", self.encoder_id )
	if not self.initialized then
		self.encoder = soc_encoder.CreateChannel( self.encoder_id )
		if self.encoder then
			if self.screen.wbcObj then
				_dbg.Info( "ENCODER", "------Ex_Init bind wbc ", self.encoder_id )
				self.mode = "wbc"
				self.encoder:BindSourceChannel( self.screen.wbcObj )
			else
				_dbg.Info( "ENCODER", "------Ex_Init bind VI ", self.encoder_id )
				self.mode = "capture"
				if self.is_sub then
					self.encoder:BindSourceChannel( _M.vi_cap_vpss1 )
				else
					self.encoder:BindSourceChannel( _M.vi_cap_vpss0 )
				end
			end
			_dbg.Info( "ENCODER", "*** Ex_Init Encoder of '", self.id, "' is working on", self.mode, "mode!" )
			self.initialized = true
		end
	end
end

function encoder_meta:Init()
	if features.DEV_BT1120_VI then
		self:Ex_Init()
	else
		if not self.initialized then	
			_dbg.Info( "ENCODER", "CreateChannel encoder_id",self.encoder_id )
			if self.screen.wbcObj then
				self.encoder = soc_encoder.CreateChannel( self.encoder_id )
				if self.encoder then
					self.mode = "wbc"
					self.encoder:BindSourceChannel( self.screen.wbcObj )
					_dbg.Info( "ENCODER", "*** Encoder of '", self.id, "' is working on", self.mode, "mode!" )
					self.initialized = true
				else
					_dbg.Error( "ENCODER", "*** Encoder of '", self.id, "' encoder CreateChannel error!" )	
				end
			else
				_dbg.Info( "ENCODER", self.encoder_id, "no wbcObj" )
				if self.encoder then
					_dbg.Info( "ENCODER", self.encoder_id, "need stop..." )
					self:Stop()
				end
			end	
		end	
	end

	self:UpdateParameters()
end

function encoder_meta:UpdateParameters()
	if self.encoder then
		local syncDetail = self.screen.syncDetail

		if self.mode == "capture" then
			capture.SetSourceVideo( syncDetail.width, syncDetail.height, syncDetail.frameRate )
		elseif self.mode == "wbc" then
			--[[
			local r,err
			r,err = self.screen.wbcObj:SetAttr({
				stTargetSize={ syncDetail.width, syncDetail.height },
				enPixelFormat = "YUV_SEMIPLANAR_420",
				u32FrameRate = syncDetail.frameRate
			})
			if not r then
				_dbg.Error( "ENCODER", "Fail to set WBC attributes:", err )
			end
			]]
		end

		self.encoder:SetSourceImage( syncDetail.width, syncDetail.height, syncDetail.frameRate )
		self.encoder.width = syncDetail.width
		self.encoder.height = syncDetail.height
		self.encoder.frameRate = syncDetail.frameRate
		local cfg = _M.encoders_config[ self.id ]
		if cfg then 
			self.encoder:SetBitrateParam( cfg.video.br_ctrl:upper(), {
				u32Gop = cfg.video.gop_size,
				u32BitRate = math.ceil( cfg.video.bitrate/1000 ),
				u32MaxBitRate = math.ceil( cfg.video.bitrate/1000 ) 
			})
			self.encoder:SetEncodingImage( cfg.video.scale_width, cfg.video.scale_height, cfg.video.framerate )
			self.encoder:SetProfile( cfg.video.profile )
		else 
			_dbg.Error( "ENCODER", "Fail to get config: ", self.id )
		end
	end
end

function encoder_meta:Start()
	if self.mode == "capture" then
		if _M.vi_cap then
			_M.vi_cap:Start()
		end

		if self.is_sub and _M.vi_cap_vpss1 then
			_M.vi_cap_vpss1:Enable()
		elseif not self.is_sub and _M.vi_cap_vpss0 then
			_M.vi_cap_vpss0:Enable()
		end
	end
	if self.encoder then
		self.encoder:Start()
	end
	self.started = true
end

function encoder_meta:Stop()
	if self.encoder then
		self.encoder:Stop()
	end
	if self.mode == "capture" then
		if self.is_sub and _M.vi_cap_vpss1 then
			_M.vi_cap_vpss1:Disable()
		elseif not self.is_sub and _M.vi_cap_vpss0 then
			_M.vi_cap_vpss0:Disable()
		end
		if _M.vi_cap then
			_M.vi_cap:Stop()
		end
	end
	self.started = nil
end

function encoder_meta:Destroy()
	self:Stop()
	self.mode = nil
	self.encoder = nil
	self.initialized = false
	collectgarbage()
end

encoder_meta.__index = encoder_meta

-------------------------------------------------------
function _M.Init()
	local cfg_encoders = require "configs.encoder_config"
	local mo_Output = require "mod_Output"
	local patch_old = false
	_M.encoders_config = cfg_encoders.Load()
	if features.FEATURE_LOCAL_STREAM then
		--TODO: Fix old version configurations.
		if not _M.encoders_config["LOCAL-00:sub"] then
			_M.encoders_config["LOCAL-00:sub"] = {}
			patch_old = true
		end
		if not _M.encoders_config["LOCAL-01:sub"] then
			_M.encoders_config["LOCAL-01:sub"] = {}
			patch_old = true
		end
		if not _M.encoders_config["LOCAL-02:sub"] then
			_M.encoders_config["LOCAL-02:sub"] = {}
			patch_old = true
		end
		if not _M.encoders_config["LOCAL-03:sub"] then
			_M.encoders_config["LOCAL-03:sub"] = {}
			patch_old = true
		end
		if not _M.encoders_config["OUTPUT1"] then
			_M.encoders_config["OUTPUT1"] = {}
			patch_old = true
		end

		if not _M.encoders_config["OUTPUT1:sub"] then
			patch_old = true
			_M.encoders_config["OUTPUT1:sub"] = { video = {
				scale_width = 720,
				scale_height = 480,
				bitrate = 2000000
			}}	
		end

		if not _M.encoders_config["OUTPUT2"] then
			_M.encoders_config["OUTPUT2"] = {}
			patch_old = true
		end

		if not _M.encoders_config["OUTPUT2:sub"] then
			patch_old = true
			_M.encoders_config["OUTPUT2:sub"] = { video = {
				scale_width = 720,
				scale_height = 480,
				bitrate = 2000000
			}}	
		end

		_M.encoders_config["OUTPUT1-00"] = nil
		_M.encoders_config["OUTPUT1-01"] = nil
		_M.encoders_config["OUTPUT2-00"] = nil
		_M.encoders_config["OUTPUT2-01"] = nil

		if patch_old then
			_M.encoders_config:TAB_SAVE()
			_M.encoders_config:TAB_RESET_CHANGE()
		end
	else
		if not _M.encoders_config then
			_dbg.Error('ENCODER', 'Fail to load encoders configure file!')
			_M.encoders_config = {
				__TRIGGER = function()
				end
			}
		end
	end

	--目前 hi3536 只能支持一个Wbc回写通道。
	local hd0 = mo_Output.all_outputs["hd:0"]

	--主码流编码
	local output1_main_enc = {
		initialized = false,
		encoder_id = 1,
		id = "OUTPUT1",
		name = "flow.output1",
		max_width = 1920,
		max_height = 1080,
		screen = hd0.screen,
		encoder = nil
	}

	setmetatable( output1_main_enc, encoder_meta )

	_M.local_out_encoders[ "OUTPUT1" ] = output1_main_enc

	--子码流编码
	local output1_sub_enc = {
		initialized = false,
		is_sub = true,
		encoder_id = 2,
		id = "OUTPUT1:sub",
		name = "flow.output1",
		max_width = 1280,
		max_height = 720,
		screen = hd0.screen,
		encoder = nil
	}
	setmetatable( output1_sub_enc, encoder_meta )
	_M.local_out_encoders[ "OUTPUT1:sub" ] = output1_sub_enc

	local hd1 = mo_Output.all_outputs["hd:1"]

	--主码流编码
	local output2_main_enc = {
		initialized = false,
		encoder_id = 3,
		id = "OUTPUT2",
		name = "flow.output2",
		max_width = 1920,
		max_height = 1080,
		screen = hd1.screen,
		encoder = nil
	}

	setmetatable( output2_main_enc, encoder_meta )

	_M.local_out_encoders[ "OUTPUT2" ] = output2_main_enc

	--子码流编码
	local output2_sub_enc = {
		initialized = false,
		encoder_id = 4,
		is_sub = true,
		id = "OUTPUT2:sub",
		name = "flow.output2",
		max_width = 1280,
		max_height = 720,
		screen = hd1.screen,
		encoder = nil
	}
	setmetatable( output2_sub_enc, encoder_meta )

	_M.local_out_encoders[ "OUTPUT2:sub" ] = output2_sub_enc

	if features.DEV_BT1120_VI then
		_M.vi_cap = capture.CreateChannel(1)
		if not _M.vi_cap then
			_dbg.Warn( "ENCODER", "Fail to create OUTPUT capture channel, so OUTPUT encoding streams may be wrong!" )
		else
			_M.vi_cap:Init()
			if not _M.vi_cap.initialized then
				_dbg.Warn( "ENCODER", "Fail to initialize OUTPUT capture channel, so OUTPUT encoding streams may be wrong!" )
				_M.vi_cap = nil
			else
				_M.vi_cap_vpss_group = _M.vi_cap:GetVPSSGroup()
				if not _M.vi_cap_vpss_group then
					_dbg.Warn( "ENCODER", "Fail to get OUTPUT capture VPSS channel, so OUTPUT encoding streams may be wrong!" )
					_M.vi_cap = nil
				else
					_M.vi_cap_vpss0 = _M.vi_cap_vpss_group:GetChannel(0)
					_M.vi_cap_vpss1 = _M.vi_cap_vpss_group:GetChannel(1)
				end
			end
		end
	end

	_M.UpdateEncoders() --Force update.
end

function _M.SetEncoder(idr, args)
	if not idr or type(args) ~= "table" then
		return false, "Invalid arguments"
	end

	local pre_cfg = _M.encoders_config[idr]
	if not pre_cfg then
		return false, "Only LOCAL-[00-03][:sub] or OUTPUT[1,2][:sub] supported"
	end

	--[[ Don't change this.
	if args.RefFrame and args.RefFrame ~= pre_cfg.video.ref_frames then
	pre_cfg.video.ref_frames = args.RefFrame
	end
	--]]

	if args.GOPSize then
		pre_cfg.video.gop_size = tonumber(args.GOPSize)
	end

	if args.BRCtrl then
		pre_cfg.video.br_ctrl = args.BRCtrl:lower()
	end

	if args.Chrome then
		pre_cfg.video.chrome = tonumber(args.Chrome)
	end

	if args.Profile then
		pre_cfg.video.profile = args.Profile:lower()
	end

	if args.Bitrate then
		pre_cfg.video.bitrate = tonumber(args.Bitrate)
	end

	if args.Scaling then
		pre_cfg.video.scale = args.Scaling
	end

	if args.FrameRate then
		pre_cfg.video.framerate = tonumber(args.FrameRate)
	end

	if args.FrameRateMode then
		pre_cfg.video.framerate_mode = args.FrameRateMode
	end

	if args.Quality then
		pre_cfg.video.quality = tonumber(args.Quality)
	end

	_M.encoders_config:__TRIGGER(true)

	local encoder = _M.local_out_encoders[idr]
	if encoder then
		encoder:UpdateParameters()
	end
	
	if args.AudioSource then
		_M.SelectAudioSource( idr, args.AudioSource )
	end

	return true
end

function _M.GetEncoder(idr)
	if not idr then
		return nil, "Invalid arguments"
	end

	local pre_cfg = _M.encoders_config[idr]
	if not pre_cfg then
		return nil, "Only LOCAL-[00-03][:sub] or OUTPUT[1,2][:sub] supported"
	end

	if not pre_cfg.video then
		_dbg.Error( "ENCODER", "Get encoders_config error!!!")
		return nil, "Get encoders_config error!!!"
	end

	return {
		GOPSize = pre_cfg.video.gop_size,
		BRCtrl = pre_cfg.video.br_ctrl,
		Chrome = pre_cfg.video.chrome,
		Profile = pre_cfg.video.profile,
		Bitrate = pre_cfg.video.bitrate,
		Scaling = pre_cfg.video.scale,
		FrameRate = pre_cfg.video.framerate,
		FrameRateMode = pre_cfg.video.framerate_mode,
		Quality = pre_cfg.video.quality,
		RefFrame = pre_cfg.video.ref_frames,
		AudioSource = pre_cfg.audio.source,
		Resolution = pre_cfg.video.scale_width.."x"..pre_cfg.video.scale_height
	}
end

function _M.UpdateEncoders()
	--local k,v
	local mode_changed = false
	for k,v in pairs(_M.local_out_encoders) do
		if not v.mode then
			_dbg.Info( "ENCODER", "*** Initialize the encoders..." )
			mode_changed = true
			break
		elseif v.mode == "wbc" and (not v.screen or not v.screen.wbcObj) then
			_dbg.Info( "ENCODER", "*** Output screen capture mode is changed! Reconfig the encoders..." )
			mode_changed = true
			break
		end
	end

	_dbg.Info( "ENCODER", "UpdateEncoders mode_changed:",mode_changed )

	-- local pt = require "pl.pretty"
	-- pt.dump(_M.local_out_encoders)
	if mode_changed then
		for k,v in pairs(_M.local_out_encoders) do
			v:Destroy()
		end

		for k,v in pairs(_M.local_out_encoders) do
			--_dbg.Info( "ENCODER", "init start" )
			v:Init()
			v:Start()

			if _M.local_out_encoders[ k ].encoder then 
				_dbg.Info( "ENCODER", " out encoder init success" )
			else 
				_dbg.Info( "ENCODER", " out encoder init fail!!! " )
			end
		end
	else
		for k,v in pairs(_M.local_out_encoders) do
			v:UpdateParameters()
		end
	end

	--NOTE D300等系列产品
	if features.FEATURE_IPStream then
		_dbg.Info( "ENCODER", "UpdateEncoders CreateStreams" )
		_M.CreateStreams()
	end
end


function _M.GetEncoderStatus(idr)
	if not idr then
		return nil, "Invalid arguments"
	end

	local encoderstatus = _M.local_out_encoders[idr]
	if not encoderstatus then
		return nil, "Only LOCAL-[00-03][:sub] or OUTPUT[1,2][:sub] supported"
	end	

	return encoderstatus
end

function _M.SelectAudioSource(idr, src)
	if not idr then
		return false, "Invalid arguments"
	end

	local pre_cfg = _M.encoders_config[idr]
	if not pre_cfg then
		return false, "Only LOCAL-[00-03][:sub] or OUTPUT[1,2][:sub] supported"
	end

	if src and src ~= pre_cfg.audio.source then
		pre_cfg.audio.source = src
		_M.encoders_config:__TRIGGER(true)
	end

	local streams = require "mod_Streams"
	local stream = streams.GetStreamObject(idr)
	if stream and type(stream.SelectAudioSource) == "function" then
		stream:SelectAudioSource( pre_cfg.audio.source )
	end
	return true
end

function _M.GetAudioSource(idr)
	if not idr then
		return nil
	end

	local pre_cfg = _M.encoders_config[idr]
	if not pre_cfg then
		return nil
	end
	return pre_cfg.audio.source
end

function _M.CreateStreams()
	local streams = require "mod_Streams"
	local id,cfg,obj,outcfg

	for id,obj in pairs(_M.local_out_encoders) do
		--obj:Init()
		--obj:Start()
		--TODO: 创建并添加流
		--NOTE D300等系列产品
		if features.FEATURE_IPStream then
			if obj.encoder then
				if streams.GetStreamObject(id) == nil then
					_dbg.Info( "ENCODER", "AddOutputStream ID:",id )
					--TODO: 创建并添加流
					if streams.AddOutputStream(id, obj.encoder_id, obj.name) then
						_dbg.Info( "ENCODER", "AddOutputStream sucess! ID:",id )
						local stream = streams.GetStreamObject(id)
						if stream and type(stream.SelectAudioSource) == "function" then				
							outcfg = _M.encoders_config[id]
							-- pt.dump(outcfg)
							if outcfg and outcfg.audio then 
								_dbg.Info( "ENCODER", "SelectAudioSource",outcfg.audio.source )
								stream:SelectAudioSource( outcfg.audio.source )
							else
								_dbg.Info( "ENCODER", "encoders_config [".. id .. "] not exist" )
							end
						end
					else
						_dbg.Info( "ENCODER", "AddOutputStream failed! ID:",id )
					end
				else
					_dbg.Info( "ENCODER", "OutputStream is exist! ID:",id )
				end
			else
				--_dbg.Info( "ENCODER", "encoder is nil! id:",id  )
				-- if streams.GetStreamObject(id) ~= nil then
				-- 	_dbg.Info( "ENCODER", "RemoveOutputStream ID:",id )
				-- 	streams.RemoveOutputStream(id)
				-- end
			end
		elseif features.DEVICE_TYPE == "E_DOCTOR" or features.DEVICE_TYPE == "E_DOCTOR_S" then
			--TODO: 创建并添加流
			if streams.AddOutputStream(id, obj.encoder_id, obj.name) then
				_dbg.Info( "ENCODER", "AddOutputStream sucess! ID:",id )
				local stream = streams.GetStreamObject(id)
				if stream and type(stream.SelectAudioSource) == "function" then				
					outcfg = _M.encoders_config[id]
					_dbg.Info( "ENCODER", "SelectAudioSource",outcfg.audio.source )
					stream:SelectAudioSource( outcfg.audio.source )
				end
			else
				_dbg.Info( "ENCODER", "AddOutputStream failed! ID:",id )
			end
		end 
		
	end

	if features.FEATURE_LOCAL_STREAM then 
		--同时，对于Amba模块，根据配置选择正确的音频源。
		for id,cfg in pairs(_M.encoders_config) do
			if id:match("^LOCAL") then --Local(Amba) modules.
				local stream = streams.GetStreamObject(id)
				if stream and type(stream.SelectAudioSource) == "function" then
					stream:SelectAudioSource( cfg.audio.source )
				end
			end
		end
	end
end

return _M


