local sys_config = require "sys_config"
local mod_Decoder = require "mod_Decoder"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local decoder = require "soc.decoder"
local aud_out = require "mod_AudioOut"
local thread = require "coaf.thread"
require "kmCore"

local _M = {
	created_streams = {}
}


local function decoderEventCallback()
	--TODO:
end

function _M.ConstructStreamMeta( stream_meta )
----------------------------------------------------------------
--{ Construct meta here!
----------------------------------------------------------------
stream_meta = stream_meta or {}

function stream_meta:Init()
	self.stream_openned = false
	self.code_info = nil

	_M.created_streams[ self.id ] = self
	--Step1: Create media synchronizer
	--_dbg.Info( "STREAM", self.id, ": Create media synchronizer, self.buffer_option:", self.buffer_option, ", audio_sync_compst:", self.audio_sync_compst )
	self.syncSource = KM_STREAMING.SyncSource.createNew()
	self.syncSource:SetBufferOptions( self.buffer_option )

	if self.SpecialInit then
		if not self:SpecialInit() then
			return false
		end
	end
	return true
end

-- Only Create DecoderChannel
function stream_meta:AllocateDecoderChannel()
	if self.decoder then
		return self.decoder
	end
	--Not decodable.
	if not self.decoderGroup then
		return nil
	end

	self.decoder_channel = mod_Decoder.AllocateDecoderChannel()
	if not self.decoder_channel then
		_dbg.Error( "STREAM", self.id, ": Fail to allocate decoder channel! Is resource limitation up?" )
		return nil
	end

	self.decoder = {}
	self.decoder.channel = self.decoder_channel - 1
	self.decoder.format = nil	--Not know the format yet
	
	return self.decoder
end

function stream_meta:CreateDecoder()
	if not self.decoder or  not self.decoderGroup or not self.decoder.channel then
		_dbg.Error( "STREAM", self.id, ": Fail create decoder!" )
	end

	if self.code_info and not self.decoder.dec then
		self.decoder.dec = decoder.CreateChannel(self.decoder_channel)
		local attrs = {}
		attrs.codec = self.code_info.codec_type or "H264"
		attrs.width = self.code_info.width or 1920
		attrs.height = self.code_info.height or 1080
		attrs.video_delay = self.code_info.has_b_frame and 1 or  0
		attrs.ref_frames =  self.code_info.ref_frames or 2
		if tonumber(attrs.ref_frames) > 16 then
			attrs.ref_frames = 2
		end
		self.decoder.dec:Configure(attrs)
		self.decoder.dec:Init()
	end

	if not self.decoder.audioSink then
		self.decoder.audioSink = aud_out.GetChannelSink( self.id )
		if not self.decoder.audioSink then
			_dbg.Error( "STREAM", self.id, ": Fail to create audio sink!" )
			return false
		end
	end

	if not self.decoder.audioGrp then
		self.decoder.audioGrp = KM_STREAMING.Audio.Decoder.createNew( "decoder-" .. self.decoder_channel )
		if not self.decoder.audioGrp then
			aud_out.FreeChannelSink( self.id )
			self.decoder.audioSink = nil
			_dbg.Error( "STREAM", self.id, ": Fail to create audio decoder!" )
			return false
		end
	end

	return true
end

--TODO
function stream_meta:RebuidDecoder()
	if type(self.decoder) ~= "table" then
		return false
	end

	if self.decoder.event_ticket then
		_M.eventObj:Unsubscribe( self.decoder.event_ticket )
		self.decoder.event_ticket = nil
	end

	if self.decoder.session then
		self.decoderGroup:RemoveSession( self.decoder.session )
		self.decoder.session = nil
	end

	if self.decoder.dec then
		self.decoder.dec:Destroy()
		self.decoder.dec = nil
	end
end

function stream_meta:FreeDecoder()
	if type(self.decoder) == "table" and self.decoder.dec then
		self.decoder.dec:Destroy()
		decoder:DestroyChannel(self.decoder.dec.id)
	end

	if type(self.decoder) == "table" then
		if self.decoder_channel then
			mod_Decoder.FreeDecoderChannel( self.decoder_channel )
			self.decoder_channel = nil
		end

		if self.decoder.audioSession then
			_dbg.Info( "STREAM", self.id, ": Destroied audio decoder" )
			self.decoder.audioGrp:StopSession( self.decoder.audioSession )
			self.decoder.audioSession = nil
			self.decoder.audioGrp:RemoveAllSessions()
			self.decoder.audioGrp = nil
		end

		if self.decoder.audioSink then
			aud_out.FreeChannelSink( self.id )
			self.decoder.audioSink = nil
		end
	end
	self.decoder = nil
	self.code_info = nil
	self.output_start = false
	self.channel = nil
	self.vpss_using = {}
end

function stream_meta:Destroy()
	self:Stop()
	self:FreeDecoder()
	if self.syncSource then
		self.syncSource:BindMediaSource(nil)
		self.syncSource = nil
	end

	if self.SpecialDestroy then
		self:SpecialDestroy()
	end
	_M.created_streams[ self.id ] = nil
	
	mod_Decoder.FreeDecoderGroup( self.decoder_group_id )

	self.start_actions = {}
	self.audio_outputs = {}
	self.audio_enabled = {}
	self.current_audio_intfs = {}
	collectgarbage()
end

function stream_meta:CheckStartActions()
	local rslt = {}
	local withActions = false
	local group, actions
	for group, actions in pairs(self.start_actions) do
		local act, v
		for act, v in pairs(actions) do
			rslt[act] = true
			withActions = true
		end
	end
	return withActions, rslt
end

function stream_meta:SetBufferOptions(opt)
	if opt and opt ~= "" then
		self.buffer_option = opt
	end

	_dbg.Info( "STREAM", self.id, "Change the buffer options:", opt )

	if self.syncSource then
		self.syncSource:SetBufferOptions( self.buffer_option )
	end
end

-- 音频同步补偿
-- 参数为毫秒
function stream_meta:SetAudioSynchroCompensation(milsec)
	_dbg.Info( "STREAM", self.id, "Change audio sync compst:", milsec ," milliSecond" )

	if self.syncSource then
		self.syncSource:SetAudioSynchroCompensation( milsec )
	end
end

function stream_meta:SetAudioOutputs(group,intfs)
	local mod_Output = require "mod_Output"
	local has_outputs = false
	local old_intfs = self.current_audio_intfs
	local k,items

	local drops = {}
	local news = {}

	if group then
		_dbg.Info( "STREAM", self.id, "SetAudioOutputs in group:", group )
		local new_intfs = {}
		local force_stop
		if nil == intfs or type(intfs) == "table" then
			self.audio_outputs[group] = intfs
		elseif intfs == "stop" then
			force_stop = true
		elseif intfs ~= "start" then
			if intfs then
				self.audio_enabled[group] = true
			else
				self.audio_enabled[group] = false
			end
		end
		for k,items in pairs(self.audio_outputs) do
			local corresponding_out = mod_Output.GetOutputByName(k)
			if not force_stop and self.audio_enabled[k] then
				local i,item
				for i,item in pairs(items) do
					if self.type ~= "local" or not corresponding_out then
						new_intfs[ item ] = true
						has_outputs = true
					elseif corresponding_out.CheckLocalAudioEnable(item) then
						new_intfs[ item ] = true
						has_outputs = true
					end
				end
			end
		end

		local item
		for k,item in pairs(old_intfs) do
			if not new_intfs[k] then
				table.insert( drops, k )
			end
		end
		for k,item in pairs(new_intfs) do
			if not old_intfs[k] then
				table.insert( news, k )
			end
		end
	elseif intfs then --Just updating
		local item
		for k,item in pairs(old_intfs) do
			table.insert( news, k )
			has_outputs = true
		end
	else --Remove all
		local item
		for k,item in pairs(old_intfs) do
			table.insert( drops, k )
			has_outputs = false
		end
	end

	for k=1, #drops do
		_dbg.Info( "STREAM", "Audio mixer:", self.id, "--X-->", drops[k] )
		self.current_audio_intfs[ drops[k] ] = nil
		aud_out.DisconnectChannelOutput( self.id, drops[k] )
	end
	for k=1, #news do
		_dbg.Info( "STREAM", "Audio mixer:", self.id, "----->", news[k] )
		self.current_audio_intfs[ news[k] ] = true
		aud_out.ConnectChannelOutput( self.id, news[k] )
	end

	if not has_outputs then
		if self.decoder and self.decoder.audioSession then
			_dbg.Info( "STREAM", self.id, ": Destroied audio decoder" )
			self.decoder.audioGrp:StopSession( self.decoder.audioSession )
			self.decoder.audioSession = nil
			self.decoder.audioGrp:RemoveAllSessions()
		end
	else
		local act = self:CheckStartActions()
		if not act then
			_dbg.Info( "STREAM", self.id, ": Streaming is not enabled so audio output is not possible! Url:", self.url )
			return
		end

		if not self.decoder or not self.decoder.audioGrp then
			_dbg.Info( "STREAM", self.id, ": Decoder is not created/enabled so audio output is not possible! Url:", self.url )
			return
		end

		if not self.decoder.audioSession then
			self.decoder.audioSession = self.decoder.audioGrp:AddSession( self.syncSource:GetMediaSource(), {
				sink = self.decoder.audioSink,
				syncIdr = "decoder-channel" .. self.decoder.channel
			})
			if not self.decoder.audioSession then
				_dbg.Warn( "STREAM", self.id, ": Fail to create audio decoding session!" )
			else
				_dbg.Info( "STREAM", self.id, ": Audio decoder started." )
				self.decoder.audioGrp:StartSession( self.decoder.audioSession )
			end
		end
	end
end

function stream_meta:Start(group, action)
	local new_with_act, new_actions
	if group then
		if not action or action == "" or action == "all" then
			self.start_actions[group] = { stream=true, output=true }
		else
			if not self.start_actions[group] then
				self.start_actions[group] = { [action] = true }
			else
				self.start_actions[group][action] = true
			end
		end
	end
	new_with_act, new_actions = self:CheckStartActions()
	if not new_with_act then
		return true
	end

	local stream_start = false
	local output_start = false

	for act, _ in pairs(new_actions) do
		if act == "output" then
			stream_start = true
			output_start = true
			break
		else
			stream_start = true
		end
	end

	if stream_start and not self.stream_openned then
		if self.SpecialStart then
			self:SpecialStart()
		end
		self.stream_openned = true
	end

	self.output_start = output_start
	if output_start then
		if not self:AllocateDecoderChannel() then
			_dbg.Error( "STREAM", self.id, ": Fail to allocate Decoder channel so could not start video output, for url:", self.url )
			return false
		end

		self:CreateDecoder()

		if self.decoder and self.decoder.dec and self.decoder.channel then
			if not self.decoder.session then
				if _M.eventObj then
					self.decoder.event_ticket = _M.eventObj:Subscribe( "decoder_notify." .. self.decoder.channel, decoderEventCallback )
					_dbg.Info( "STREAM", self.id, ": Subscribed decoder event of 'decoder_notify." .. self.decoder.channel .. "'" )
				end
				self.decoder.dec:Start()
				local errMsg
				self.decoder.session, errMsg = self.decoderGroup:AddSession( self.syncSource:GetMediaSource(), {
					syncIdr="decoder-channel" .. self.decoder.channel,
					event= _M.eventName,
					topic="decoder_notify." .. self.decoder.channel,
					channel = self.decoder.channel
				})

				if not self.decoder.session then
					_dbg.Error( "STREAM", self.id, ": Start output - fail to create Hi-Decoder session, error:", errMsg )
					return false
				else
					_dbg.Info( "STREAM", self.id, ": Start output - decoder is started." )
				end
				self.decoderGroup:StartSession( self.decoder.session )
			end
		end
		self:SetAudioOutputs(group, "start")
	end
	return true
end

function stream_meta:CheckMediaCodecInfo()
	if self.syncSource and not self.code_info  and self.output_start and self.stream_openned then
		self.code_info = self.syncSource:GetCodeInfo()
		if self.code_info then
			local pt = require "pl.pretty"
			pt.dump(self.code_info)
			--note: 此处必须调用Update去重新绑定视频输出。由于update会重新拉流 故不需要特意去创建解码器
			local mo_output = require "mod_Output"
			coaf.ScheduleDelayedTask(function()
				mo_output.UpdateStreamBinding()
			end, 0)
		end
	end
end

function stream_meta:Stop(group, action)
	local new_with_act, new_actions
	if group then
		if not action or action == "" or action == "all" then
			self.start_actions[group] = nil
		else
			if self.start_actions[group] then
				self.start_actions[group][action] = nil
			end
		end
	end

	local stream_start = false
	local output_start = false

	if group then
		new_with_act, new_actions = self:CheckStartActions()
		if new_with_act then
			local act, v
			for act, v in pairs(new_actions) do
				if act == "output" then
					stream_start = true
					output_start = true
					break
				else
					stream_start = true
				end
			end
		end
	end

	if not output_start and self.decoder and self.decoder.session then
		if self.decoder.event_ticket then
			_M.eventObj:Unsubscribe( self.decoder.event_ticket )
			self.decoder.event_ticket = nil
		end
		self.decoderGroup:RemoveSession( self.decoder.session )
		self.decoder.session = nil
	end

	if not output_start then
		self:SetAudioOutputs( group, "stop" )
		if group or not action then --if not group and action == true, That don't free decoder because it will start again.
			self:FreeDecoder()
		end
	end

	if not stream_start and self.stream_openned then
		if self.SpecialStop then
			self:SpecialStop()
		end
		self.stream_openned = false
	end

	self.output_start = output_start
	return true
end


function stream_meta:EnableAudioOutput(group, en)
	return self:SetAudioOutputs(group,en)
end

function stream_meta:GetAudioOutputEnable(group)
	if not group then
		local k,v
		local fnd_group
		for k,v in pairs(self.audio_enabled) do
			if v then
				fnd_group = k
				break
			end
		end
		if not fnd_group then return false end
		return self.audio_outputs[fnd_group] and #self.audio_outputs[fnd_group] > 0 or false
	else
		if not self.audio_enabled[group] then
			return false
		else
			return self.audio_outputs[group] and #self.audio_outputs[group] > 0 or false
		end
	end
end

function stream_meta:GetDisplayVpss(group)
	return self:GetVpss(group)
end

function stream_meta:GetVpss(group)
	if self.decoder and self.decoder.dec then
		group = group or "default"

		local free_vpss_chns = { 0,1,2 }
		if self.vpss_using[group] then
			return self.vpss_using[group].vpss
		end
		local k,v 
		for k,v in pairs(self.vpss_using) do
			table.remove( free_vpss_chns, v.channel_id+1 )
		end

		local vpss_chn
		if group == "preview" or group == "decoder" then
			vpss_chn = 3
		elseif #free_vpss_chns == 0 then
			_dbg.Error( "STREAM", "GetVpss: All VPSS channels are busy so no valid VPSS channel for group:", group, ",Url:", self.url )
			return nil
		else
			vpss_chn = free_vpss_chns[1]
		end

		local vpssGrp = self.decoder.dec:GetVPSSGroup()
		if not vpssGrp then
			_dbg.Error( "STREAM", "GetVpss: No VPSS grouup so no valid VPSS channel for group:", group, ",Url:", self.url )
			return nil
		else
			local vpss = vpssGrp:GetChannel(vpss_chn)
			self.vpss_using[group] = { channel_id = vpss_chn, vpss = vpss }
			return vpss
		end
	else
		return nil
	end
end

stream_meta.__index = stream_meta

return stream_meta
----------------------------------------------------
-- } End of construct meta.
----------------------------------------------------
end

local for500msTimer
local flag_code_updating = false

local function _for500ms_PeriodCheck()
	for _, obj in pairs(_M.created_streams) do
		-- 只有打开了预览的流采取检测信息
		if obj and not flag_code_updating then
			flag_code_updating = true
			obj:CheckMediaCodecInfo()
			flag_code_updating = false
		end
	end
end


function _M.Init(event, event_name)
	_M.eventObj = event
	_M.eventName = event_name

	for500msTimer = thread.NewTimer( _for500ms_PeriodCheck, 0.01, 0.5 )
	if for500msTimer then
		for500msTimer:start()
	end
end

function _M.CheckStreamExist(id)
	return _M.created_streams[id]
end

return _M

