local pt = require("pl.pretty")
local _dbg = require "coaf.debug"

local _M = {
	layouts_config = nil,
	id_base = 1
}

local grids = {
	["1"] = {
		{ x=0, y=0, w=1, h=1 }
	},

	["2"] = {
		{ x=0, y=0.25, w=0.5, h=0.5 },
		{ x=0.5, y=0.25, w=0.5, h=0.5 }
	},

	["3"] = {
		{ x=0, y=0.25, w=0.5, h=0.5 },
		{ x=0.5, y=0, w=0.5, h=0.5 },
		{ x=0.5, y=0.5, w=0.5, h=0.5 }
	},

	["4"] = {
		{ x=0, y=0, w=0.5, h=0.5 },
		{ x=0.5, y=0, w=0.5, h=0.5 },
		{ x=0, y=0.5, w=0.5, h=0.5 },
		{ x=0.5, y=0.5, w=0.5, h=0.5 }
	},

	["6"] = {
		{ x=0, y=0, w=2/3, h=2/3 },
		{ x=2/3, y=0, w=1/3, h=1/3 },
		{ x=2/3, y=1/3, w=1/3, h=1/3 },
		{ x=0, y=2/3, w=1/3, h=1/3 },
		{ x=1/3, y=2/3, w=1/3, h=1/3 },
		{ x=2/3, y=2/3, w=1/3, h=1/3 }
	},

	["9"] = {
		{ x=0, y=0, w=1/3, h=1/3 },
		{ x=1/3, y=0, w=1/3, h=1/3 },
		{ x=2/3, y=0, w=1/3, h=1/3 },
		{ x=0, y=1/3, w=1/3, h=1/3 },
		{ x=1/3, y=1/3, w=1/3, h=1/3 },
		{ x=2/3, y=1/3, w=1/3, h=1/3 },
		{ x=0, y=2/3, w=1/3, h=1/3 },
		{ x=1/3, y=2/3, w=1/3, h=1/3 },
		{ x=2/3, y=2/3, w=1/3, h=1/3 }
	},

	["16"] = {
		{ x=0, y=0, w=1/4, h=1/4 },
		{ x=1/4, y=0, w=1/4, h=1/4 },
		{ x=2/4, y=0, w=1/4, h=1/4 },
		{ x=3/4, y=0, w=1/4, h=1/4 },
		{ x=0, y=1/4, w=1/4, h=1/4 },
		{ x=1/4, y=1/4, w=1/4, h=1/4 },
		{ x=2/4, y=1/4, w=1/4, h=1/4 },
		{ x=3/4, y=1/4, w=1/4, h=1/4 },
		{ x=0, y=2/4, w=1/4, h=1/4 },
		{ x=1/4, y=2/4, w=1/4, h=1/4 },
		{ x=2/4, y=2/4, w=1/4, h=1/4 },
		{ x=3/4, y=2/4, w=1/4, h=1/4 },
		{ x=0, y=3/4, w=1/4, h=1/4 },
		{ x=1/4, y=3/4, w=1/4, h=1/4 },
		{ x=2/4, y=3/4, w=1/4, h=1/4 },
		{ x=3/4, y=3/4, w=1/4, h=1/4 }
	}
}

function _M.Init()
	local layout_configs = require "configs.layouts_config"
	_M.layouts_config = layout_configs.Load()
	if not _M.layouts_config then
		_dbg.Error( "LAYOUT", "Fail to load layout configure file!" )
		_M.layouts_config = {
			__TRIGGER = function() end
		}
	end

	local max_id, id
	for k, set in pairs(_M.layouts_config) do
		id = tonumber(k)	
		if not max_id or id > max_id then
			max_id = id
		end		
	end		

	_M.id_base = (max_id or 0) + 1
end

function _M.AddLayout( layoutobj )
	if layoutobj then
		local id = _M.id_base
		layoutobj.id = id
		_M.layouts_config[tostring(id)] = layoutobj
		_M.layouts_config:__TRIGGER(true)
		_M.id_base = _M.id_base + 1 
	end
	return true
end

function _M.ChangeLayout( id, layoutobj )
	if not id then
		_dbg.Error( "LAYOUT", "Changing layout but without valid ID!" )
		return false
	end

	if	(not layoutobj.id)	then
		layoutobj.id = id
	end

	if ( type(id) == "number" ) then
		id = tostring(id)
	end

	

	if(_M.layouts_config[id]) then
		_M.layouts_config[id] = layoutobj
		_M.layouts_config:__TRIGGER(true)
	end
	return true
end

function _M.RemoveLayout( id )
	if not id then
		_dbg.Error( "LAYOUT", "Removing layout but without valid ID!" )
		return false
	end

	if ( type(id) == "number" ) then
		id = tostring(id)
	end

	_M.layouts_config[id] = nil
	_M.layouts_config:__TRIGGER(true)
	collectgarbage()	
	return true
end


function _M.EnumLayouts()
	local r = {}
	local k,v
	for k,v in pairs(_M.layouts_config) do
		table.insert( r, k )
	end
	-- table.sort(r, function(a,b)
	-- 	return a < b 
	-- end)
	return r
end



--	{x = 70, y  = 70, width = 30,	height = 30 }
-- 	{ x=0.7, y=0.7, w=0.3, h=0.3 }



function _M.GetLayoutDetail(id)	
	if not id then
		_dbg.Error( "LAYOUT", "GetLayoutDetail but without valid ID!" )
		return nil
	end


	if ( type(id) == "number" ) then
		id = tostring(id)
	end

	if not _M.layouts_config[id] then
		_dbg.Error( "LAYOUT", "GetLayoutDetail ID not exist! id = ",id )
		return nil
	end

	if not _M.layouts_config[id].position then
		_dbg.Error( "LAYOUT", "GetLayoutDetail position not exist!" )
		return nil
	end

	local layout = {}
	
	local newlayout = _M.layouts_config[id].position


	--	{ x=70, y=70, width=30,	height=30 }
	--  changeto...
	-- 	{ x=0.7, y=0.7, w=0.3, h=0.3 }
	if newlayout then
		local k,v
		
		for k,v in pairs(newlayout) do
			local wininfo = {
				x=v.x/100,
				y=v.y/100,
				w=v.width/100,
				h=v.height/100,
			}
			table.insert( layout, wininfo )
		end
	end
	return layout
end

function _M.GetAllLayouts()
	return _M.layouts_config
end


function _M.GetLayout( id )	
	if id then
		return _M.layouts_config[id]
	end
	return nil
end

return _M

