--[[
0: 没有磁盘
1: 挂载正常，没录象
2：没有挂载
3：正在录像
4：异常
5: 磁盘空间不足
6: 磁盘正在初始化

]]
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local event = require "coaf.event"
local thread = require "coaf.thread"

local lfs = require "lfs"
local LED = require "mod_diskled"
local streams = require "mod_Streams"

local CACHE_OUTTIME = 30

local _M = {
	file_system = "NTFS",
	disk_event = nil,
	disk_event_name = nil,
	cached_disks_info = nil,
	last_update_disks_info_time = nil,
	init_status = {},
	disk_status = {[1] = { is_record = 0, name = nil, status = 0, is_remove = false}, [2] = { is_record = 0, name = nil, status = 0, is_remove = false}}
}

local FS_SUPPORT = {}

FS_SUPPORT["NTFS"] = {
	mount = function(dev,point)
		local string_exe = "/usr/bin/ntfs-3g /dev/" .. dev .. " " .. point
		return os.execute( string_exe )
	end,

	format = function(dev)
		return os.execute( "/sbin/mkfs.ntfs -Q /dev/" .. dev ) 
	end
}

FS_SUPPORT["VFAT"] = {
	mount = function(dev,point)
		return os.execute( "/bin/mount -t vfat /dev/" .. dev .. " " .. point )
	end,

	format = function(dev)
		return os.execute( "/sbin/mkfs.vfat -F 32 /dev/" .. dev )
	end
}

FS_SUPPORT["EXT4"] = {
	mount = function(dev,point)
		return os.execute( "/bin/mount -t ext4 /dev/" .. dev .. " " .. point )
	end,

	format = function(dev)
		return os.execute( "/bin/mkfs.ext4 -T largefile /dev/" .. dev ) --Yes, it's in /bin not /sbin
	end
}

local function for125ms_PeriodCheck()
	--_G.TIME_TICK = _G.TIME_TICK + 0.125
	_G.TIME_TICK =  0.125
	--DetectSDISignal() --NOTE: Now control the LED status by mainApp.
	LED.Step()
end


function create_led_service()
	LED.Init()
    
    LED.CreateLight( "RED1", { --GPIO4_0
        muxreg = 0x120f016c,
        muxval = 1,
        pingroup = 0x12190000,
        pin = 0
    })

    LED.CreateLight( "BLUE1", { --GPIO4_1
        muxreg = 0x120f0170,
        muxval = 1,
        pingroup = 0x12190000,
        pin = 1
    })

    LED.CreateLight( "RED2", { --GPIO4_2
        muxreg = 0x120f0174,
        muxval = 1,
        pingroup = 0x12190000,
        pin = 2
    })

    LED.CreateLight( "BLUE2", { --GPIO4_3
        muxreg = 0x120f0178,
        muxval = 1,
        pingroup = 0x12190000,
        pin = 3
    })	

    -- LED.SetLightStatus( "RED1", LED.BLINK)
    -- LED.SetLightStatus( "BLUE1", LED.BLINK_VERYFAST)
    -- LED.SetLightStatus( "RED2", LED.BLINK)
    -- LED.SetLightStatus( "BLUE2", LED.BLINK)

    for125msTimer = coaf.NewTimer( for125ms_PeriodCheck, 0.01, 0.125 )
	if for125msTimer then
		for125msTimer:start()
	end

end

function load_disk()
	create_led_service()
	local disk_info = _M.GetDiskDetail(nil, true)
	_M.disk_status[1].name = disk_info[1].dev 
	_M.disk_status[2].name = disk_info[2].dev
end

local led_status = {[0] ={RED = LED.OFF, BLUE = LED.OFF}, [1] = {RED = LED.OFF, BLUE = LED.ON}, [2] = {RED = LED.OFF, BLUE = LED.BLINK_VERYFAST},
					[3] = {RED = LED.OFF, BLUE = LED.BLINK}, [4] = {RED = LED.ON, BLUE = LED.OFF}, [5] = {RED = LED.BLINK, BLUE = LED.OFF},
				    [6] = {RED = LED.OFF, BLUE = LED.OFF} }

function change_disk_status(disk_id, cur_status)
	--print("change disk status disk id is "..disk_id.." cur status is "..cur_status.." LED.BLINK is "..LED.BLINK.." led_status[5][\"RED\"] is "..led_status[5]["RED"]..
	--" led_status[3][\"BLUE\"] is "..led_status[3]["BLUE"])
	if _M.disk_status[disk_id].disk_status ~= cur_status then
		_M.disk_status[disk_id].disk_status = cur_status
		LED.SetLightStatus( "RED"..disk_id, led_status[cur_status]["RED"])
		LED.SetLightStatus( "BLUE"..disk_id, led_status[cur_status]["BLUE"])
		--to control disk led

	end
end

function _M.change_disk_status(disk_id, cur_status)
	 change_disk_status(disk_id, cur_status)
end

function _M.Init(event,name)
	_M.disk_event = event
	_M.disk_event_name = name

	load_disk()
end

local function onDiskInitReport(topic,msg,msgType)
	if msg and msgType == "table" and msg.id then
		local init_item = _M.init_status[msg.id]
		if init_item then
			init_item.status = msg.status or init_item.status
			init_item.message = msg.message or init_item.message
			init_item.percent = msg.percent or init_item.percent
		end
	end
end

local function waitTask(threadObj,  disk_id, pid, init_item)
	print("come into waitTask############################################")
	_M.GetDiskDetail(disk_id, true)
	local r,err = coaf.WaitPID(pid) 
	if r then
		if init_item.status == "init" then
			init_item.status = "done"      
		end
		init_item.pid = nil
		init_item.waitThread = nil
	else
		if init_item.pid then os.execute( "/bin/kill " .. init_item.pid ) end

		init_item.status = "error"
		init_item.message = "error_exit"
		init_item.waitThread = nil
		init_item.pid = nil
	end
	print("init finish==================")
	_M.GetDiskDetail(disk_id, true)
	_M.last_update_disks_info_time = 0	--So mark to update disk info.
	coaf.ScheduleDelayedTask(function()
		local record = require "mod_record"
		for k, v in pairs(record.recording) do
			if v.status and tonumber(v.status.use_disk) and tonumber(v.status.use_disk) == tonumber(disk_id) then
				record.recording[k] = nil
			end
		end
	end, 0)
end


local function read_sys_arg( path )
	local f = io.open( path, "r" )
	if not f then
		return nil
	else
		local ln = f:read("*l")
		f:close()
		return ln
	end
end

function _M.EnumDisks()
    local disks = {[1] = {online = false}, [2] = {online = false} }
	local blkname
	local disk_id
    for blkname in lfs.dir( "/sys/class/block" ) do
		if blkname:match("sd[a-z]$") then

			local readfile = io.popen("readlink -f /sys/class/block/" .. blkname)
			local content = readfile:read("*all")
			local result1 = string.match(content, "/sys/devices/platform/ahci.0/ata1.*")
			local result2 = string.match(content, "/sys/devices/platform/ahci.0/ata2.*")

			if result1  or result2 then

				if result1 then
					disk_id = 1
				else
					disk_id = 2
				end

				disks[disk_id].online = true
				disks[disk_id].name = blkname
				disks[disk_id].size_sectors = read_sys_arg( "/sys/class/block/" .. blkname .. "/size" )--Keep in string format, since LUA number may overflow.
				disks[disk_id].partitions = {}
				
				
				
				

				local partname
				for partname in lfs.dir( "/sys/class/block/" .. blkname ) do
					if partname:match( blkname .. "%d+" ) then
						local part_size = read_sys_arg( "/sys/class/block/" .. blkname .. "/" .. partname .. "/size" )
						if part_size then
							table.insert( disks[disk_id].partitions, {
								name = partname,
								id = tonumber( read_sys_arg("/sys/class/block/" .. blkname .. "/" .. partname .. "/partition" ) ),
								size = part_size --Keep in string format
							})
						end
					end
				end

				table.sort( disks[disk_id].partitions, function(a,b)
					return a.name < b.name
				end)
			end
        end
    end

	
    return disks
end

local function CheckMount( dev )
	local rslt
	if not dev:match("%d$") then
		rslt = {}
	end
	local f = io.open( "/proc/mounts", "r" )
	if f then
		local l = f:read("*l")
		while l do
			local m, point = l:match("^/dev/(" .. dev .. ".-)%s+(.-)%s")
			if m then
				if not rslt then
					rslt = point 
					break
				else
					rslt[m] = point
				end
			end
			l = f:read("*l")
		end
		f:close()
	end
	return rslt
end



local function MountDisk( dev )
	if not dev:match("%d$") then
		dev = dev .. "1"	--TODO: Now just only check partition1
	end
	local r = CheckMount(dev)
	if r then
		return r
	end

	os.execute( "/bin/mkdir -p /media/disks/" .. dev )
	FS_SUPPORT[ _M.file_system ].mount( dev, "/media/disks/" .. dev )
	return CheckMount(dev)
end

local function UmountDisk( dev )
	local r = CheckMount(dev)
	if type(r) == "table" then
		local k,v
		for k,v in pairs(r) do
			local _ = os.execute( "/bin/umount -l /dev/" .. k )
		end
	elseif type(r) == "string" then
		local _ = os.execute( "/bin/umount -l /dev/" .. dev )
	end
end

local function UmountAll()
	local r = _M.EnumDisks()
	local i
	for i=1, #r do
		UmountDisk(r[i].name)
	end
end

local function GetDiskSize(dev)
	local file=io.popen("/bin/df -m /dev/" .. dev,"r")
	if not file then           
		return nil
	end

	local line = file:read("l")
	while line do
		local flag,t,u,f,p = line:match( "(.-)%s+(%d+)%s+(%d+)%s+(%d+)%s+(%d+)%%%s+.*$" )
		if flag then
			file:close()
			return tonumber(t), tonumber(u), tonumber(f), tonumber(p)
		end

		line=file:read("l")
	end
	file:close()
	return nil
end


local function GetDiskFormate(dev)


end

function _M.GetDiskDetail( disk_id, force_update )
	local rslt
	disk_id = tonumber(disk_id)
	if not disk_id then
		rslt = {}
	end

	local now = os.time()
	if force_update or not _M.cached_disks_info or not _M.last_update_disks_info_time 
		or (now-_M.last_update_disks_info_time) >= CACHE_OUTTIME 
		or (now-_M.last_update_disks_info_time) < 0 then
		_M.cached_disks_info = _M.EnumDisks()
		_M.last_update_disks_info_time = now
	end

	local i
	local r_item
	local disk_status
	for i=1, #_M.cached_disks_info do
		if not disk_id or disk_id == i then
			local disk_item = _M.cached_disks_info[i]
			if disk_item.online then
				if _M.disk_status[i].is_remove then
					r_item = {online = true, is_remove = true}
					disk_status = 2
				else
					r_item = {
						online = true,
						is_remove = false,
						id = i,
						name = "SATA" .. i,
						dev = disk_item.name,
						total = 0,
						free = 0,
						used = 0,
						percent = 100,
						status = "bad",
						mount = "",
						progress = "not_init",
						init_percent = 0
					}

					if _M.init_status[i] and _M.init_status[i].status and _M.init_status[i].status ~= "done" then
						if _M.init_status[i].status == "error" then
							disk_status = 4
							r_item.progress = _M.init_status[i].message
							r_item.init_percent = _M.init_status[i].percent
						else
							r_item.status = "init"
							--r_item.progress = _M.init_status[i].message .. " - " .. _M.init_status[i].percent .. "%"
							r_item.progress = _M.init_status[i].message
							--r_item.init_percent = _M.init_status[i].percent	
							if _M.init_status[i].percent < 80 then
								_M.init_status[i].percent = _M.init_status[i].percent + 8	--Fake progress
								r_item.init_percent = _M.init_status[i].percent
								--_M.disk_status[i].status = 2
								--change_disk_status(i, {status = 2})
							else 
								r_item.init_percent = _M.init_status[i].percent	
							end
							disk_status = 6
						end
					elseif disk_item.partitions and disk_item.partitions[1] then
						local mpoint = MountDisk( disk_item.partitions[1].name )
						if mpoint then
							r_item.mount = mpoint
							local t,u,f,p = GetDiskSize( disk_item.partitions[1].name ) 
							if t then
								r_item.status = "ready"
								r_item.total = math.floor(t/1024)
								r_item.free = math.floor(f and (f/1024) or 0)
								r_item.used = math.floor(u and (u/1024) or 0)
								r_item.percent = p or 100

								if r_item.free < 1 then
									disk_status = 5
								else
									disk_status = 1
								end
							else
								disk_status = 4

							end

							
						else
							disk_status = 4
						end

					else
						disk_status = 4
					end
				end
			else	
				r_item = {online = false}
				disk_status = 0
			end
			if _M.disk_status[i].is_record ~= 0 and disk_status == 1 then
				disk_status = 3
			end

			
			change_disk_status(i, disk_status)

			if rslt then
				table.insert(rslt, r_item)
			else
				return r_item
			end
		end
	end
	return rslt
end

local function actionInitDisk(topic, id, dev)
	_M.disk_event:NotifyBack( topic, { id = id, status = "init", message = "rebuilding", percent = 10 } )
	local r,status,code = os.execute( "/bin/cat /usr/local/bin/codec/preset/init_fdisk." .. _M.file_system .. ".cmd | /sbin/fdisk /dev/" .. dev )
	if not r or status ~= "exit" or code ~= 0 then
		print("**************fdisk   error*******************")
		_M.disk_event:NotifyBack( topic, { id = id, status = "error", message = "partition_failed", percent = 100 } )
		return
	end
	_M.disk_event:NotifyBack( topic, { id = id, status = "init", message = "formating", percent = 50 } )
	--coaf.Sleep(3)
	os.execute("sleep "..3)
	r,status,code = FS_SUPPORT[ _M.file_system ].format( dev .. "1" )
	if not r or status ~= "exit" or code ~= 0 then
		_M.disk_event:NotifyBack( topic, { id = id, status = "error", message = "format_failed:" .. (status or "FAIL") .. "(" .. (code or -1) .. ")", percent = 100 } )
		return
	end
	_M.disk_event:NotifyBack( topic, { id = id, status = "init", message = "init_success", percent = 100 } )
	--coaf.Sleep(5)
	os.execute("sleep "..3)
	_M.disk_event:NotifyBack( topic, { id = id, status = "done", message = "", percent = 100 } )
end

function _M.InitDisk( disk_id )
	_M.cached_disks_info = _M.EnumDisks() --Force update.
	_M.last_update_disks_info_time = os.time()
	
	disk_id = tonumber(disk_id)
	local i
	for i=1, #_M.cached_disks_info do
		if not disk_id or disk_id == i then
			local disk_item = _M.cached_disks_info[i]
			if not disk_item.online then
			
				return false, "this disk no exitence"
			elseif disk_item.status == 3 then
				return false, "this disk is recording"

			end
			local status_info = _M.init_status[i]
			if not status_info then
				status_info = {
					status = "done",
					message = "",
					percent = 100
				}
				_M.init_status[i] = status_info
			end

			if status_info.waitThread then
				_dbg.Warn( "STREAMER", "Disk #" .. i .. " is initialzing on progress while requiring initialize it!" )
			else
				status_info.status = "init"
				status_info.message = "prepare_init"
				status_info.percent = 0

				--check filestream
				streams.StopPlayBeforeInitDisk(disk_item.name)
				--Umount disk first.  
				UmountDisk( disk_item.name )

				local topic = "init_disk." .. i
				_M.disk_event:Subscribe( topic, onDiskInitReport )
				local pid = coaf.Fork(function()
					actionInitDisk(topic, i, disk_item.name)
					os.exit(0)
				end)

				if pid and pid > 0 then
					status_info.pid = pid
					status_info.waitThread = thread.New( waitTask )
					status_info.waitThread:Start(disk_id, pid, status_info)
					
				else --Can't start init, so make it ready.
					status_info.status = "done"
					status_info.message = ""
					status_info.percent = 100
				end
			end
		end
	end
	return true
end

function _M.GetValidDisks()
	local r = _M.GetDiskDetail(nil,true)
	local rslt = {}
	local i
	for i=1, #r do
		if r[i].online and r[i].status == "ready" and r[i].mount and r[i].mount ~= '' then
			table.insert( rslt, {
				name = r[i].name,
				mount = r[i].mount,
				total = r[i].total,
				free = r[i].free,
				id = r[i].id
			})
		end
	end
	return rslt
end

function _M.PrepareRecord()
	--TODO: nothing.
end






function _M.add_disk(dev)
	print("come into -----------------------------add disk"..dev)
	if not string.match(dev, "%d$") then
		_dbg.Warn( "STREAMER", "dev-"..dev.." not disk partition" )
		return false
	end

	local rel_dev = string.sub(dev, 1, 3)
	if not rel_dev then
		_dbg.Error("STREAMER", "add disk unknow error......")
		return false
	end
	_M.cached_disks_info = _M.EnumDisks()
	local i
	for i=1, #_M.cached_disks_info do
		if _M.cached_disks_info[i].online and _M.cached_disks_info[i].name == rel_dev then
			if not (_M.init_status[i] and _M.init_status[i].status and _M.init_status[i].status == "init")   then
				_M.init_status[i] = nil
			end
			_M.disk_status[i].name = rel_dev
			_M.disk_status[i].is_remove = false

			local disk_info = _M.GetDiskDetail( i, true )
		end
	end



end

function _M.kill_disk(dev)
	print("----------kill disk "..dev)
	UmountDisk(dev)

	local rel_dev = string.sub(dev, 1, 3)
	if not rel_dev then
		_dbg.Error("STREAMER", "kill disk unknow error......")
		return false
	end
	print ("rel_dev is-----------: "..rel_dev)

	for i = 1, #_M.disk_status do
		if _M.disk_status[i].name == rel_dev then
			_M.disk_status[i].name = nil
			_M.disk_status[i].is_record = 0
			if not (_M.init_status[i] and _M.init_status[i].status and _M.init_status[i].status == "init") then
				-- _M.disk_status[i].status = 0
				_M.init_status[i] = nil
				print ("update disk status***********************************************")
				_M.GetDiskDetail( i, true )
			end
			

		end

	end

end

function _M.remove_disk(disk_id)

	if not _M.disk_status[disk_id] then
		return false, "this disk is not exitest"
	end
	if _M.disk_status[disk_id].is_record ~= 0 then
		return false, "this disk is using record now"
	end
	local disk_info = _M.GetDiskDetail( disk_id, true )
	if disk_info.online then
		UmountDisk(disk_info.dev)
		-- _M.disk_status[disk_id].status = 2

		_M.init_status[disk_id] = nil
	   -- _M.disk_status[disk_id].name = nil
		_M.disk_status[disk_id].is_record = 0
		_M.disk_status[disk_id].is_remove = true

		--print("remove disk id is :"..disk_id.." disk status is 2")
		--change_disk_status(disk_id, 2)

		_M.GetDiskDetail( disk_id, true )
		
	else
		return false,  "this disk is not exitest"
	end
	return true
end

function _M.get_disk_status()
	local sata_status = {}
	sata_status[1] = {status = _M.disk_status[1].status}
	sata_status[2] = {status = _M.disk_status[2].status}
	return sata_status
end


function _M.get_disk_info()
	local disk_info =  _M.GetDiskDetail( nil, false )
	local i
	for i=1, #disk_info do
		--if disk_info[i].online then
			disk_info[i].specific_status = _M.disk_status[i].disk_status
		--end
	end
	return disk_info

end

return _M

