local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local periphery = require "hiperiphery"
local I2C = periphery.I2C
local HIMM = periphery.HIMM

local _M = {
	i2c_dev = "/dev/i2c-0",
	BASE_ADDR = 0xa4,
	i2c_handle = nil,
}

function _M.Init()
	local done,err = pcall( function() _M.i2c_handle = I2C( _M.i2c_dev ) end )
	if not done then
		if type(err) == "table" then
			_dbg.Error( "FPGA_CONTROL", "Could not open I2C device for FPGA controlling. ErrMsg:", err.message or "---" )
		else
			_dbg.Error( "FPGA_CONTROL", "Could not open I2C device for FPGA controlling. ErrMsg:", err )
		end
	else
		_dbg.Info( "FPGA_CONTROL", "FPGA controller openned, at " .. string.format("%02x",_M.BASE_ADDR) .. "@" .. _M.i2c_dev )
	end
end

function _M.ReadReg( addr, itemArr )
	if not _M.i2c_handle then
		return nil, "Invalid I2C handle"
	end
	if not itemArr or type(itemArr) ~= "table" then
		itemArr = { 0xff, flags = I2C.I2C_M_RD }
	else
		if #itemArr < 1 then
			itemArr[1] = 0xff
		end
		if not itemArr.flags then
			itemArr.flags = I2C.I2C_M_RD
		else
			itemArr.flags = itemArr.flags | I2C.I2C_M_RD
		end
	end

	local addrs = {}
	local i
	for i=1, #itemArr do
		addrs[i] = addr+i-1
	end

	local msg = { addrs, itemArr }
	local done,err = pcall( function() _M.i2c_handle:transfer( _M.BASE_ADDR>>1, msg ) end )
	if not done then
		if type(err) == "table" then
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err )
			return nil, err
		end
	else
		return itemArr
	end
end

-----------------------------------

function _M.WriteReg( addr, itemArr )
	if not _M.i2c_handle then
		return false, "Invalid I2C handle"
	end
	if not itemArr or type(itemArr) ~= "table" or #itemArr < 1 then
		return false, "Invalid data"
	end

	--table.insert( itemArr, 1, addr )
	local addrs = {}
	local i
	for i=1, #itemArr do
		addrs[i] = addr+i-1
	end
	local msg = { addrs, itemArr }
	local done,err = pcall( function() _M.i2c_handle:transfer( _M.BASE_ADDR>>1, msg ) end )
	if not done then
		if type(err) == "table" then
			--print( string.format( "WriteReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--print( string.format( "WriteReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err )
			return nil, err
		end
	else
		return true
	end
end

return _M

