require "himpp"
local vo_drivers = require "video.vo_drivers"
local _dbg = require "coaf.debug"
local pt = require "pl.pretty"

local SYS = HiMPP.SYS
local VPSS = HiMPP.VPSS
local VO = HiMPP.VO

--add by kiloview@2020-9-28
local _done, features = pcall(require, 'features')
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

----------------------------
local _M = {
	--[[ Should be like this:
	{
		["sd"] = { 0 },
		["hd"] = { 0, 1 }
	}]]
	support_dev_types = nil,
	outputs = {},
	vpss_channel_modes = {}
}

----------------------------

function _M.Init()
	vo_drivers.Init()
	_M.support_dev_types = vo_drivers.GetSoCSupportDevices()

	local vodev
	local k,items,i,v
	for k,items in pairs(_M.support_dev_types) do
		if k == "hd" then
			for i,v in ipairs(items) do
				vodev = VO.GetHDDevice(v)
				vodev:ForceFree()
			end
		elseif k == "sd" then
			for i,v in ipairs(items) do
				vodev = VO.GetSDDevice(v)
				vodev:ForceFree()
			end
		end
	end
	vodev = nil
	collectgarbage()
end

--

function _M.Cleanup()
	local k,o
	for k,o in pairs(_M.outputs) do
		o:Destroy()
	end
	_M.outputs = {}
end

function _M.SetVpssChannelUserMode( vpss_chn, chanObj, output, osdsize )
	if not vpss_chn or not chanObj then
		return
	end

	local vpss_idr = vpss_chn.DID .. ":" .. vpss_chn.ID

	local map = _M.vpss_channel_modes[ vpss_idr ]
	if not map then
		map = {}
		_M.vpss_channel_modes[ vpss_idr ] = map
	end
	if output then
		map[ chanObj.channel_idr ] = { rect = chanObj.windowRect, frameRate = output.syncDetail and output.syncDetail.frameRate }
	else
		map[ chanObj.channel_idr ] = nil
	end

	local max_w = 0
	local max_h = 0
	local max_fr = 0

	local chn_idr, out_rec
	for chn_idr, out_rec in pairs(map) do
		if out_rec.rect and out_rec.rect[3] > max_w then
			max_w = out_rec.rect[3]
			max_h = out_rec.rect[4]
		end
		if out_rec.frameRate and out_rec.frameRate > max_fr then
			max_fr = out_rec.frameRate
		end
	end
	
	-- print("chanObj.channel_idr~~~~~~~~~~~~~~~~~~~~~",chanObj.channel_idr)
	-- if chanObj.channel_idr == "hd:0:0" or chanObj.channel_idr == "hd:1:0" then
	if osdsize then
		max_w = osdsize.w &0xfffffffe	--必须是2的倍数
		max_h = osdsize.h &0xfffffffe
	end

	if max_w > 0 and max_h > 0 then
		local r, err = vpss_chn:SetMode({
			enChnMode = "USER",
			u32Width = max_w,
			u32Height = max_h,
			bDouble = false,

			stFrameRate = { --TODO: How to update this?
				s32SrcFrmRate = max_fr, s32DstFrmRate = max_fr
			},

			stAspectRatio = {
				enMode = "NONE", u32BgColor = 0, stVideoRect = {0,0,0,0}
			},

			enPixelFormat = "YUV_SEMIPLANAR_420",
			enCompressMode = "NONE"
		})
		if not r then
			_dbg.Error( "OUTPUT", "Fail to Set VPSS Channel Mode for ID=" .. vpss_idr .. ", error:", err )
		end
	end
end

function _M.RemoveVpssChannelUserMode( vpss_chn, chanObj )
	_M.SetVpssChannelUserMode( vpss_chn, chanObj, nil )
end

----------------------------

local out_meta = {}

------
function out_meta:UpdateDisplay(forceDestroy)
	_dbg.Info( "OUTPUT", self.name, "UpdateDisplay: first disable VO device:", string.format("%08X",
	self.voDevice:Disable()
	))

	--NOTE: Here, for our FPGA SDI output case, we should do some fake things: If output
	--      resolution is PAL/NTSC (SD resolutions), actually, I need output to 1080i50/
	--      1080i60, put the image at left-top corner (as size 720x576 or 720x480), then
	--      tell FPGA crop the corner image to output to SD resolution. So, before set
	--      the VO attributes, I need check and translate the resolution to perform right 
	--      action.

	local k,v,drv
	local translated_res

	local Enablewbc = false

	--Check all binding interfaces.
	for k,v in pairs( self.deviceAttr.enIntfType ) do
		_dbg.Info( "OUTPUT", self.name, "UpdateDisplay bind interface -->", v )
		drv = vo_drivers.GetInterfaceDriver(v)
		if drv and drv.TranslateResolution then --Got the driver, so need update VO settings.
			--NOTE: if actually need translate, then this will return a resolution,
			--      otherwise, it will return nil.
			translated_res = drv:TranslateResolution( self.resolution )
		end
		if v == "NET" then
			Enablewbc = true
			_dbg.Info( "OUTPUT", self.name, "UpdateDisplay Enable wbcObj" )
		end
	end

	if Enablewbc == true then
		self.EnablewbcObj = true
	else
		self.EnablewbcObj = false
	end

	if translated_res then
		self.deviceAttr.enIntfSync = translated_res
	end

	if features.FEATURE_IPStream  then  --D300系列产品
		-- local pt = require "pl.pretty"
		-- pt.dump(self.deviceAttr)
		local tmparr = self.deviceAttr.enIntfType
		if #tmparr == 1 and tmparr[1] == "NET" then
			tmparr[1] = "VGA"
		end
		--pt.dump(self.deviceAttr)
	end

	local r,err
	r, err = self.voDevice:SetPubAttr( self.deviceAttr )
	if not r then
		_dbg.Error( "OUTPUT", self.name, "UpdateDisplay: Fail to set output device attributes! reason:", err )
		if forceDestroy then
			self:RemoveChannel()	--So remove all.
			self.voLayer = nil
			self.voDevice = nil
		end
		return false
	end

	--NOTE: you MUST enable VO device to setup layer attributes.
	r = self.voDevice:Enable()
	if r ~= 0 then
		_dbg.Error( "OUTPUT", self.name, "UpdateDisplay: Fail to enable VO device, with error:", string.format("%08X",r) )
	end

	local k,v,drv
	for k,v in pairs( self.deviceAttr.enIntfType ) do
		drv = vo_drivers.GetInterfaceDriver(v)
		if drv then --Got the driver, so need update VO settings.
			drv:UpdateResolution( self.resolution, self.syncDetail, self.t1001 )
		end
	end
	
	-- if features.DEVICE_TYPE == "E_DOCTOR" or features.DEVICE_TYPE == "E_DOCTOR_S" then
	--NOTE here. 置视频层的分割模式 默认是MULTI 要设置边框、背景图需要配置SINGLE模式
		self.voLayer:SetPartitionMode( "SINGLE" )
	-- end

	local layerAttr = {
		stDispRect = { 0, 0, self.syncDetail.width, self.syncDetail.height },
		stImageSize = { self.syncDetail.width, self.syncDetail.height },
		u32DispFrmRt = self.syncDetail.frameRate,
		enPixFormat = "YUV_SEMIPLANAR_420",
		bDoubleFrame = false,
		bClusterMode = false
	}

	r, err = self.voLayer:SetAttr( layerAttr )
	if not r then
		_dbg.Error( "OUTPUT", self.name, "UpdateDisplay: Fail to set output layer attributes! reason:", err )
		if forceDestroy then
			self:RemoveChannel() --So remove all.
			self.voLayer = nil
			self.voDevice = nil
		end
		return false
	end

	_dbg.Info( "OUTPUT", self.name, "UpdateDisplay: DONE." )
	return true
end

---------

function out_meta:Init()
	_dbg.Info( "OUTPUT", "output init", self.initialized )
	if self.initialized then
		return
	end

	self.EnablewbcObj = false

	if self.devType == "hd" then
		self.voDevice = VO.GetHDDevice(self.devId)
	else
		self.voDevice = VO.GetSDDevice(self.devId)
	end

	if not self.voDevice then
		_dbg.Error( "OUTPUT", self.name, "Init: Fail to get video output device!" )
		return
	end

	self.voLayer = self.voDevice:GetLayer()

--  NOTE: No need bind since it's static bond.
--	local r, err
--	r,err = self.voLayer:Bind( self.voDevice )
--	if not r then
--		_dbg.Error( "OUTPUT", "Fail to bind layer to VO device:", err )
--	end

	if not self:UpdateDisplay( true ) then
		return
	end

	self.initialized = true
end

------

function out_meta:Destroy()
	if self.initialized then
		if self.wbcObj then
			self.wbcObj:Disable()
		end
		self:RemoveChannel() --Remove all channels.

		self:UnbindAllInterfaces(true) --For release.

		self.voDevice:Disable()
		self.voLayer:Disable()
		
		self.wbcObj = nil
		self.voChannels = nil
		self.voLayer = nil
		self.voDevice = nil

		self.initialized = false
	end
end

function out_meta:BindInterface( iftype )
	_dbg.Info( "OUTPUT", self.name, "BindInterface....", iftype)
	if not iftype then
		return false
	end

	local ifExist
	local i
	for i=1, #self.deviceAttr.enIntfType do
		if features.FEATURE_IPStream and not features.DEV_BT1120_VI  then --D300系列产品
			if self.deviceAttr.enIntfType[i] == iftype or (self.deviceAttr.enIntfType[i] == "VGA" and iftype == "NET")  then 
				ifExist = i
				break
			end
		else
			if self.deviceAttr.enIntfType[i] == iftype then 
				ifExist = i
				break
			end
		end
	end

	if ifExist then
		return true
	end

	local supports = vo_drivers.GetSupportInterfaces()
	local exact_if = supports[iftype]
	if not exact_if then
		_dbg.Error( "OUTPUT", self.name, "BindInterface: Invalid video output interface type of:", iftype )
		return false
	end

	if self.devType ~= exact_if.deviceType then
		_dbg.Error( "OUTPUT", self.name, "BindInterface:", iftype, " can not bind with current output device:", self.devType, ",id:", self.devId )
		return false
	end

	if exact_if.deviceId and exact_if.deviceId ~= self.devId then
		_dbg.Error( "OUTPUT", self.name, "BindInterface:", iftype, " can only bind with device:", exact_if.deviceType, ",id:", exact_if.deviceId, "but current output is device:", self.devType, ",id:", self.devId )
		return false
	end

	iftype = exact_if.ifname
	local user = vo_drivers.CheckInterfaceUsing( iftype )
	if user then
		_dbg.Error( "OUTPUT", self.name, "BindInterface:", iftype, " is used by other outputs -", user )
		return false
	end

	table.insert( self.deviceAttr.enIntfType, iftype )
	vo_drivers.MarkInterfaceUsing( iftype, self.name )

	local started = self.started
	if started then
		self:Stop()
	end

	if self.initialized then
		self:UpdateDisplay()
	end
	
	if started then
		self:Start()
	end

	return true
end

function out_meta:UnbindInterface( iftype )
	if not iftype then
		return false
	end

	local supports = vo_drivers.GetSupportInterfaces()
	local exact_if = supports[iftype]
	if not exact_if then
		_dbg.Error( "OUTPUT", self.name, "UnbindInterface: Invalid video output interface type of:", iftype )
		return false
	end
	
	iftype = exact_if.ifname

	local ifExist = nil
	local i
	for i=1, #self.deviceAttr.enIntfType do
		if features.FEATURE_IPStream and not features.DEV_BT1120_VI  then --D300系列产品
			if self.deviceAttr.enIntfType[i] == iftype or (self.deviceAttr.enIntfType[i] == "VGA" and iftype == "NET")  then 
				ifExist = i
				break
			end
		else
			if self.deviceAttr.enIntfType[i] == iftype then 
				ifExist = i
				break
			end
		end
	end

	if not ifExist then
		return true
	end

	--table.remove( self.deviceAttr.enIntfType, ifExist ) --NOTE hhy
	vo_drivers.MarkInterfaceUsing( iftype, nil )

	local drv = vo_drivers.GetInterfaceDriver(iftype)
	if drv then
		drv:Disable()
	end

	local started = self.started
	if started then
		self:Stop()
	end
	
	table.remove( self.deviceAttr.enIntfType, ifExist ) --NOTE hhy
	if self.initialized then
		self:UpdateDisplay()
	end

	if started then
		self:Start()
	end

	return true
end

function out_meta:UnbindAllInterfaces(forRelease)
	local i,iftype
	local drv
	for i,iftype in ipairs(self.deviceAttr.enIntfType) do
		drv = vo_drivers.GetInterfaceDriver(iftype)
		if drv then
			drv:Disable()
		end
		vo_drivers.MarkInterfaceUsing( iftype, nil )
	end

	if forRelease then
		return true
	end

	self.deviceAttr.enIntfType = {}
	local started = self.started
	if started then
		self:Stop()
	end

	if self.initialized then
		self:UpdateDisplay()
	end
	
	if started then
		self:Start()
	end

	return true
end

-- local function float_equal(lhs, rhs, epsilon)
--     local abs= math.abs
--     epsilon  = epsilon or 1E-12
--     return abs(lhs - rhs) <= epsilon * (abs(lhs) + abs(rhs))
-- end

local function float_equal(lhs, rhs, epsilon)
    epsilon  = epsilon or 1E-12

    local abs= math.abs
    local abslhs = abs(lhs)
    local absrhs = abs(rhs)
    local diff = abs(abslhs-absrhs)

    if (lhs == rhs) then 
        return true
    elseif (lhs == 0 or rhs == 0 or diff < 1E-12) then        
        return diff < (epsilon * 1E-12)
    else
        -- return diff / (abslhs + absrhs) < epsilon;
        return abs(lhs - rhs) <= epsilon
    end
end

------
function out_meta:_LayoutChannel( id, chanObj )
	local channel = chanObj.channel
	local layout = chanObj.layout

	local x, y, w, h
	if ( float_equal(layout.x, 0.3333, 0.0001) ) then
		x = self.syncDetail.width / 3
	elseif  ( float_equal(layout.x, 0.6666, 0.0001) ) then
		x = self.syncDetail.width / 3 * 2
	else
		x = self.syncDetail.width * (layout.x or 0)
	end


	if ( float_equal(layout.y, 0.3333, 0.0001) ) then
		y = self.syncDetail.height / 3
	elseif  ( float_equal(layout.y, 0.6666, 0.0001) ) then
		y = self.syncDetail.height / 3 * 2
	else
		y = self.syncDetail.height * (layout.y or 0)
	end	

	if  ( float_equal(layout.w, 0.3333, 0.0001) ) then
		w = self.syncDetail.width / 3
	elseif  ( float_equal(layout.w, 0.6666, 0.0001) ) then
		w = self.syncDetail.width / 3 * 2
	else
		w = self.syncDetail.width * (layout.w or 1)
	end

	if  ( float_equal(layout.h, 0.3333, 0.0001) ) then
		h = self.syncDetail.height / 3
	elseif  ( float_equal(layout.h, 0.6666, 0.0001) ) then
		h = self.syncDetail.height / 3 * 2
	else
		h = self.syncDetail.height * (layout.h or 1)
	end	
	
	x = math.floor(x) & 0xfffc
	y = math.floor(y) & 0xfffe
	w = math.floor(w) & 0xfffc
	h = math.floor(h) & 0xfffe

	local r, err = channel:SetAttr({
		u32Priority = id,
		stRect = { x, y, w, h },
		bDeflicker = false
	})

	if not r then
		_dbg.Error( "OUTPUT", self.name, "_LayoutChannel: Fail to set output picture attribute for channel:", id, " reason:", err )
		return false
	end

	--注意：在做VO图像处理时(需要配置全局vb内存池),且需要重新设置windowRect
	chanObj.windowRect = { x, y, w, h }

	_dbg.Info( "OUTPUT", self.name, "_LayoutChannel: Set output picture attribute for channel:", id, "DONE." )
	return true
end

function out_meta:CreateChannel( id, layout )
	if not self.initialized then
		_dbg.Error( "OUTPUT", self.name, "CreateChannel: output object is not initialized!" )
		return false
	end

	id = tonumber(id)
	if not id then
		_dbg.Error( "OUTPUT", self.name, "CreateChannel: Fail to create channel since without valid ID!" )
		return false
	end

	if not layout then
		layout = { x=0, y=0, w=1, h=1 }
	end

	if self.voChannels[id] then
		_dbg.Warn( "OUTPUT", self.name, "CreateChannel: channel", id, " is alreay exist! (Just set the new layout configure)" )
		self.voChannels[id].layout = layout
		return true
	end

	local chanObj = {
		channel_idr = self.name .. ":" .. id,
		channel = self.voLayer:GetChannel(id),
		src = nil,
		layout = layout
	}

	self.voChannels[id] = chanObj
	chanObj.channel:Disable()	--If already enabled?
	self:_LayoutChannel( id, chanObj )
	return true
end

function out_meta:GetVoChannel( id )
	return self.voChannels[id]
end

function out_meta:RemoveChannel( id )
	id = tonumber(id)
	local i, chanObj
	for i, chanObj in pairs(self.voChannels) do
		if not id or id == i then
			if chanObj.src then
				if not chanObj.dontDisableSrc then
					_dbg.Info( "OUTPUT", self.name, "RemoveChannel:", id, ", disable src channel:", string.format("%08X",
					chanObj.src:Disable()
					))
				end

				_dbg.Info( "OUTPUT", self.name, "RemoveChannel:", id, ", disable vo channel:", string.format("%08X",
				chanObj.channel:Disable()
				))

				SYS.UnBind( chanObj.src, chanObj.channel )
				chanObj.channel = nil
			end

			if id then
				self.voChannels[id] = nil
				collectgarbage()
				return true
			end
		end
	end

	self.voChannels = {}
	collectgarbage()
	return true
end

local function setBorder(vpsschannel, enable, bordercolor)
	_dbg.Info( "OUTPUT", "setBorder")
	if not enable then
		enable = false
	end

	if not bordercolor then
		bordercolor = 0xffffff
	end

	local stBorder = {
		u32TopWidth = 2,
		u32BottomWidth = 2,
		u32LeftWidth = 2,
		u32RightWidth = 2,
		u32Color = bordercolor
	}
	local ret
	local attr,err = vpsschannel:GetAttr()
	if (attr and type(attr) == "table") then
		attr.bBorderEn = enable
		attr.stBorder = stBorder

		ret,err = vpsschannel:SetAttr(attr)
		--_dbg.Info( "OUTPUT", "Vpss SetAttr done ret=", ret," err=", err)
	else
		_dbg.Error( "OUTPUT", "Vpss GetBorder error, ret=", attr," err=", err)
	end		
end


local function setVoBorder(vochannel, enable, bordercolor)
	if not enable then
		enable = false
	end

	if not bordercolor then
		bordercolor = 0xffffff
	end

	local stBorder = {
		u32TopWidth = 2,
		u32BottomWidth = 2,
		u32LeftWidth = 2,
		u32RightWidth = 2,
		u32Color = bordercolor
	}
	local ret = vochannel:SetBorder(enable, stBorder)
	if ret < 0 then
		_dbg.Error( "OUTPUT", "VO SetBorder error ret=", string.format( "0x%02x",ret))	
	else  
		_dbg.Info( "OUTPUT", "VO SetBorder OK ret=", ret)	
	end
end


function out_meta:SetVpssBorder( enable, bordercolor, id ) --SetVpssBorder -> SetVoBorder
	_dbg.Info( "OUTPUT", "1 SetVpssBorder id=", id)	
	if enable == 1 then
		enable = true
	else
		enable = false
	end
	id = tonumber(id)
	local i, chanObj
	for i, chanObj in pairs(self.voChannels) do
		_dbg.Info( "OUTPUT", "2 SetVpssBorder id=", id, ",i:", i)	
		if not id or id == i then
			local fullscreen = false
			if chanObj.windowRect and chanObj.windowRect[1] == 0 and chanObj.windowRect[2] == 0 and chanObj.windowRect[3] == self.syncDetail.width and chanObj.windowRect[4] == self.syncDetail.height then
				fullscreen = true
			end
			if chanObj.src then
				if not fullscreen then
					setVoBorder(chanObj.channel, enable, bordercolor)
				else	
					setVoBorder(chanObj.channel, false, bordercolor)
				end
			end

			if (id and self.voChannels[id])then	
				if not fullscreen then
					setVoBorder(self.voChannels[id].channel, enable, bordercolor)
				else
					setVoBorder(self.voChannels[id].channel, false, bordercolor)
				end
				return true
			end
		end
	end
	return true
end

function out_meta:BindChannelSource( id, src_vpss, dontDisable, osdsize )
	id = tonumber(id)
	if not id then
		_dbg.Error( "OUTPUT", self.name, "BindChannelSource: specified an invalid channel ID!" )
		return false
	end

	local chanObj = self.voChannels[id]
	if not chanObj then
		_dbg.Error( "OUTPUT", self.name, "BindChannelSource: channel", id, " is not created yet!" )
		return false
	end

	if chanObj.src ~= src_vpss or osdsize then
		if chanObj.src then
			if not chanObj.dontDisableSrc then
				_dbg.Info( "OUTPUT", self.name, "BindChannelSource: Changing channel source, channel:", id, ", disable old-src channel:", string.format("%08X",
				chanObj.src:Disable()
				))
			end
			
			_dbg.Info( "OUTPUT", self.name, "BindChannelSource: Changing channel source, channel:", id, ", disable vo channel:", string.format("%08X",
			chanObj.channel:Disable()
			))

			SYS.UnBind( chanObj.src, chanObj.channel )
			_M.RemoveVpssChannelUserMode( chanObj.src, chanObj )
		end

		chanObj.src = src_vpss

		if chanObj.src then
			--Update VPSS mode.
			_M.SetVpssChannelUserMode( chanObj.src, chanObj, self, osdsize )
			local r, err = SYS.Bind( chanObj.src, chanObj.channel )
			if not r then
				_dbg.Error( "OUTPUT", self.name, "BindChannelSource: Fail to bind VPSS channel to output channel:", id, "! reason:", err )
			end

			if self.started then
				chanObj.channel:Enable()

				if chanObj.src.Enable then
					chanObj.src:Enable()
				end
			end
		end
	end
	chanObj.dontDisableSrc = dontDisable
	return true
end

function out_meta:ReLayout(forDisable)
	if not self.initialized then
		return
	end

	--NOTE: Enable layer first.[**MUST**, otherwise the layout attributes can't be set]
	if not forDisable then
		_dbg.Info( "OUTPUT", self.name, "ReLayout: so first enable vo-layer:", string.format("%08X",
		self.voLayer:Enable()
		))
	end

	local i, chanObj
	for i,chanObj in pairs(self.voChannels) do
		if --[[forDisable and]] chanObj.src then
			if not chanObj.dontDisableSrc then
				_dbg.Info( "OUTPUT", self.name, "ReLayout: channel", i, " disable src channel:", string.format("%08X",
				chanObj.src:Disable()
				))
			end
		end

		_dbg.Info( "OUTPUT", self.name, "ReLayout: channel", i, " disable vo channel:", string.format("%08X",
		chanObj.channel:Disable()
		))

		--Should unbind? Tested: no needed.
		--if forDisable and chanObj.src then
		--	SYS.UnBind( chanObj.src, chanObj.channel )
		--end

		if not forDisable then
			self:_LayoutChannel( i, chanObj )
		end
	end

	if self.started and not forDisable then
		for i,chanObj in pairs(self.voChannels) do
			if chanObj.src then
				--Should re-bind again? Tested: no needed.
				--r, err = SYS.Bind( chanObj.src, chanObj.channel )
				--if not r then
				--	_dbg.Error( "OUTPUT", "Fail to bind VPSS channel to output channel for '" .. self.name .. "', channel:", id, "! reason:", err )
				--end

				--Update VPSS mode.
				--排除 background vpss
				if chanObj.channel_idr ~= "hd:0:0" and chanObj.channel_idr ~= "hd:1:0" then
					_M.SetVpssChannelUserMode( chanObj.src, chanObj, self )
				end

				--But also try enable if already bind.
				_dbg.Info( "OUTPUT", self.name, "ReLayout: Enable channel:", i, string.format("%08X",
				chanObj.channel:Enable()
				))

				if chanObj.src.Enable then
					chanObj.src:Enable()
				end
			end
		end
	end
end

------

function out_meta:Start(iftype)

	if not self.initialized then
		self:Init()
	end

	if self.initialized and not self.started then
		--NOTE here, I must mark started be true, either voLayer:Enable() be fail.
		self.started = true

		local r
		r = self.voLayer:Enable()
		if r ~= 0 then
			_dbg.Error( "OUTPUT", self.name, "Start: Enable Layer:", string.format("%08X", r) )
			return
		end

		self:ReLayout()

		if self.EnablewbcObj then
			_dbg.Info( "OUTPUT", "CreateWbc" )
			self:CreateWbc()
		else
			_dbg.Info( "OUTPUT", "no CreateWbc" )
			self:ReleaseWbc()
		end

		local k,v,drv
		for k,v in pairs( self.deviceAttr.enIntfType ) do
			drv = vo_drivers.GetInterfaceDriver(v)
			if drv then --Got the driver, so need control to start/enable.
				drv:Enable()
			end
		end
	end
end

------

function out_meta:Stop()
	if self.initialized and self.started then
		self.started = false

		local k,v,drv
		for k,v in pairs( self.deviceAttr.enIntfType ) do
			drv = vo_drivers.GetInterfaceDriver(v)
			if drv then --Got the driver, so need control to stop/disable.
				drv:Disable()
			end
		end

		self:ReLayout( true )	--For disable only.

		_dbg.Info( "OUTPUT",  self.name, "Stop: disable vo layer:", string.format("%08X",
		self.voLayer:Disable()
		))

		if self.wbcObj then
			_dbg.Info( "OUTPUT", self.name, "Stop: Disable wbc:", string.format("%08X",
				self.wbcObj:Disable()
			))
		end
	end
end

------
local function CompareResolution(a,b)
	if a == b then
		return true
	elseif not a or not b then
		return false
	else
		return a.width == b.width and a.height == b.height and a.frameRate == b.frameRate and a.interlace == b.interlace
	end
end

function out_meta:ChangeResolution(res)
	if res then
		local act_res, syncDetail,t1001 = vo_drivers.FetchBestFormat( res, self.devType, self.devId, self.deviceAttr.enIntfType )
		if not act_res then
			_dbg.Error( "OUTPUT", self.name, "ChangeResolution: Unsupported resolution:", res )
			return
		else
			_dbg.Info( "OUTPUT", self.name, "ChangeResolution: set resolution", res, ", fetched resolution is", act_res, ", t1001:", t1001 )
		end

		if act_res ~= self.resolution or t1001 ~= self.t1001 then
			self.resolution = act_res
			self.deviceAttr.enIntfSync = act_res
			self.syncDetail = syncDetail
			self.t1001 = t1001

			local started = self.started
			if started then
				self:Stop()
			end

			if self.initialized then
				self:UpdateDisplay()
			end

			if started then
				self:Start()
			end
		end
	end
end

------

function out_meta:CreateWbc()
	if self.devType ~= "hd" then
		_dbg.Error( "OUTPUT", self.anme, "CreateWbc: Now only support create Wbc on digital(hd) output!" )
		return nil
	end

	if not self.wbcObj then
		_dbg.Info("OUTPUT", "CreateWbc:VO.GetWBC")
		self.wbcObj = VO.GetWBC(0)
	else
		_dbg.Info("OUTPUT", "CreateWbc:Disable")
		self.wbcObj:Disable()
	end
	
	_dbg.Info( "OUTPUT", self.name, "CreateWbc: set WBC source:", 
	self.wbcObj:SetSource( "DEVICE", self.voDevice )
	)

	local r,err
	r,err = self.wbcObj:SetAttr({
		stTargetSize={ self.syncDetail.width, self.syncDetail.height },
		enPixelFormat = "YUV_SEMIPLANAR_420",
		u32FrameRate = self.syncDetail.frameRate
	})
	if not r then
		_dbg.Error( "OUTPUT", self.name, "CreateWbc: Fail to set attributes:", err )
		--NOTE here: Keep it.
		--self.wbcObj = nil
		return nil
	end

	_dbg.Info( "OUTPUT", self.name, "CreateWbc: Initialize WBC..." )
	self.wbcObj:SetDepth(0)
	--self.wbcObj:SetMode( "NORMAL" )
	self.wbcObj:SetMode( "DROP_REPEAT" )
	

	_dbg.Info( "OUTPUT", self.name, "CreateWbc: enable WBC:", string.format("%08X",
	self.wbcObj:Enable()
	))

	_dbg.Info( "OUTPUT", self.name, "CreateWbc: WBC initialized." )
	return self.wbcObj
end

function out_meta:ReleaseWbc()
	_dbg.Info( "OUTPUT", self.name, "releasWbc..." )
	if self.wbcObj then
		_dbg.Info( "OUTPUT", self.name, "releasWbc Disable..." )
		self.wbcObj:Disable()
		self.wbcObj = nil
	end
end

out_meta.__index = out_meta

----------------------------

local function CheckDeviceUsing(devType,devId)
	local k,v,already_using
	for k,v in pairs(_M.outputs) do
		if v.devType == devType and v.devId == devId then
			already_using = k
			break
		end
	end
	return already_using
end

-- Get the output object

function _M.CreateOutput( name, res, bind_interface )
	if type(name) ~= "string" then
		_dbg.Error( "OUTPUT", "_M.CreateOutput: Must specify a valid output name/id, it should be 'hdmi'/'vga'/'digital'/'sdi'/'cvbs' or 'hd|sd[:0-x]'! But you are requiring '", name, "'" )
		return nil
	end

	name = name:lower()

	if _M.outputs[name] then
		_dbg.Info( "OUTPUT", "_M.CreateOutput: The output '" .. name .. "' is already created, returns the exists object." )
		return _M.outputs[name]
	end

	local devType = "hd"
	local devId = 0
	local should_bind
	local can_dynamic_sel = true

	if name == "digital" or name == "sdi" then
		devId = 1
		should_bind = "BT1120"
	elseif name == "hdmi" then
		devId = 0
		should_bind = "HDMI"
		--can_dynamic_sel = false
	elseif name == "vga" then
		devId = 1
		should_bind = "VGA"
	elseif name == "cvbs" then
		devType = "sd"
		devId = 0
		should_bind = "CVBS"
		can_dynamic_sel = false
	else
		devType, devId = name:match("(%w+):(%d+)")
		if not devType then
			devType = name
			devId = 0
		else
			devId = tonumber(devId)
		end
		can_dynamic_sel = false
	end

	local sup = _M.support_dev_types
	if not sup or not sup[devType] then
		_dbg.Error( "OUTPUT", "_M.CreateOutput: Unsupported device type:", devType, ",id:", devId, "for current SoC while creating output!" )
		return nil
	else
		local i,fnd
		for i=1, #sup[devType] do
			if sup[devType][i] == devId then
				fnd = true
				break
			end
		end
		if not fnd then
			_dbg.Error( "OUTPUT", "_M.CreateOutput: Unsupported device type:", devType, ",id:", devId, "for current SoC while creating output!" )
			return nil
		end
	end

	if not bind_interface then
		bind_interface = should_bind
	end

	if type(bind_interface) == "string" then
		bind_interface = { bind_interface:upper() }
	elseif type(bind_interface) == "nil" then
		bind_interface = {}
	end

	local already_using = CheckDeviceUsing( devType, devId )
	if already_using and not can_dynamic_sel then
		_dbg.Error( "OUTPUT", "_M.CreateOutput: The output device:", devType, ",id:", devId, "is used by:", already_using )
		return nil
	end

	if already_using then
		local _id, trying
		local fnd
		for _id, trying in pairs(_M.support_dev_types[ devType ]) do
			if trying ~= devId and not CheckDeviceUsing(devType, trying) then
				devId = trying
				fnd = true
				break
			end
		end
	
		if not fnd then
			_dbg.Error( "OUTPUT", "_M.CreateOutput: The output device:", devType, ",id:", devId, "is used and no any other same type device can be use!" )
			return nil
		end
	end

	local outObj = {
		name = name,
		voDevice = nil,
		voLayer = nil,
		devType = devType,
		devId = devId,
		voChannels = {},

		resolution = "1080P60", -- Should update later.
		t1001 = false,

		deviceAttr = {
			u32BgColor = 0,
			enIntfType = {}, --Should update later.
			enIntfSync = "1080P60" --Should update later.
		},
		syncDetail = {} --Should update later.
	}

	local r = setmetatable( outObj, out_meta )
	_M.outputs[name] = r

	local i
	for i=1, #bind_interface do
		r:BindInterface( bind_interface[i] )
	end

	local actRes,syncDetail,t1001 = vo_drivers.FetchBestFormat(res, devType, devId, r.deviceAttr.enIntfType )
	if not actRes then
		_dbg.Error( "OUTPUT", "_M.CreateOutput: Invalid resolution of '" .. res .. "' and no chance to choose a better format for it!" )
	end

	r.resolution = actRes
	r.t1001 = t1001
	r.deviceAttr.enIntfSync = actRes
	r.syncDetail = syncDetail

	return r
end

-------------------------------------

-------------------------------------

return _M

