local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local hi_hdmi = require "video.vodrv_hi_hdmi"
local kv_sdi = require "video.vodrv_fpga_sdi_kiloview_v1"
local vodrv_common = require "video.vo_drivers_common"

local _M = {
	support_4k = true,
	support_t1001 = true,
	support_hdmi_ycbcr422 = true,
	support_hdmi_ycbcr420 = true,

	--SoC supported devices (according to HiSi device special)
	socSupports = {
		["sd"] = {0},
		["hd"] = {0,1}
	},

	--VO ports using recording.
	voUsing = {},

	--VO ports supported for current product.
	voSupports = {
		HDMI = { ifname="HDMI", deviceType="hd" },
		BT1120 = { ifname="BT1120", deviceType="hd" },
		SDI = { ifname="BT1120", deviceType="hd" },
		VGA = { ifname="VGA", deviceType="hd" },
		CVBS = { ifname="CVBS", deviceType="sd" },
		NET = { ifname="NET", deviceType="hd" }
	},

	--VO driver(s) bond with speicifed VO ports.
	voDrivers = {
	}
}

_M.syncDetailMap = {
	--Default: sd=false, no_hd=false, four_k_only=false, no_sdi=false, no_hdmi=false, no_vga=false
    ["PAL"] = { name="480i/525i", width=720, height=576, frameRate=50, interlace=true, sd=true, no_vga=true } ,
    ["NTSC"] = { name="576i/625i", width=720, height=480, frameRate=60, interlace=true, sd=true, no_vga=true } ,
    ["960H_PAL"] = { name="960H", width=960, height=576, frameRate=50, interlace=false, sd=true, no_hd=true, no_vga=true, no_suggest = true } ,
    ["960H_NTSC"] = { name="960H", width=960, height=480, frameRate=60, interlace=false, sd=true, no_hd=true, no_vga=true, no_suggest = true } ,
    ["1080P24"] = { name="1080p", width=1920, height=1080, frameRate=24, interlace=false } ,
    ["1080P25"] = { name="1080p", width=1920, height=1080, frameRate=25, interlace=false } ,
    ["1080P30"] = { name="1080p", width=1920, height=1080, frameRate=30, interlace=false } ,
    ["720P50"] = { name="720p", width=1280, height=720, frameRate=50, interlace=false } ,
    ["720P60"] = { name="720p", width=1280, height=720, frameRate=60, interlace=false } ,
    ["1080I50"] = { name="1080i", width=1920, height=1080, frameRate=50, interlace=true, no_vga=true } ,
    ["1080I60"] = { name="1080i", width=1920, height=1080, frameRate=60, interlace=true, no_vga=true } ,
    ["1080P50"] = { name="1080p", width=1920, height=1080, frameRate=50, interlace=false } ,
    ["1080P60"] = { name="1080p", width=1920, height=1080, frameRate=60, interlace=false } ,
    ["576P50"] = { name="576p", width=720, height=576, frameRate=50, interlace=false, no_sdi=true } ,
    ["480P60"] = { name="480p", width=720, height=480, frameRate=60, interlace=false, no_sdi=true } ,
    ["640x480@60"] = { name="640x480@", width=640, height=480, frameRate=60, interlace=false, no_sdi=true } ,
    ["800x600@60"] = { name="800x600@", width=800, height=600, frameRate=60, interlace=false, no_sdi=true } ,
    ["1024x768@60"] = { name="1024x768@", width=1024, height=768, frameRate=60, interlace=false, no_sdi=true } ,
    ["1280x1024@60"] = { name="1280x1024@", width=1280, height=1024, frameRate=60, interlace=false, no_sdi=true } ,
    ["1366x768@60"] = { name="1366x768@", width=1366, height=768, frameRate=60, interlace=false, no_sdi=true } ,
    ["1440x900@60"] = { name="1440x900@", width=1440, height=900, frameRate=60, interlace=false, no_sdi=true } ,
    ["1280x800@60"] = { name="1280x800@", width=1280, height=800, frameRate=60, interlace=false, no_sdi=true } ,
    ["1680x1050@60"] = { name="1680x1050@", width=1680, height=1050, frameRate=60, interlace=false, no_sdi=true } ,
    ["1920x2160@30"] = { name="1920x2160@", width=1920, height=2160, frameRate=30, interlace=false, no_sdi=true, four_k_only = true } ,
    ["1600x1200@60"] = { name="1600x1200@", width=1600, height=1200, frameRate=60, interlace=false, no_sdi=true, no_suggest = true } ,
    ["1920x1200@60"] = { name="1920x1200@", width=1920, height=1200, frameRate=60, interlace=false, no_sdi=true, no_suggest = true } ,
    ["2560x1440@30"] = { name="2560x1440@", width=2560, height=1440, frameRate=30, interlace=false, no_sdi=true, four_k_only=true } ,
    ["2560x1440@60"] = { name="2560x1440@", width=2560, height=1440, frameRate=60, interlace=false, no_sdi=true, four_k_only=true } ,
    ["2560x1600@60"] = { name="2560x1600@", width=2560, height=1600, frameRate=60, interlace=false, no_sdi=true, four_k_only=true } ,
    ["3840x2160@25"] = { name="3840x2160@", width=3840, height=2160, frameRate=25, interlace=false, no_sdi=true, four_k_only=true } ,
    ["3840x2160@30"] = { name="3840x2160@", width=3840, height=2160, frameRate=30, interlace=false, no_sdi=true, four_k_only=true } ,
    ["3840x2160@50"] = { name="3840x2160@", width=3840, height=2160, frameRate=50, interlace=false, no_sdi=true, four_k_only=true } ,
    ["3840x2160@60"] = { name="3840x2160@", width=3840, height=2160, frameRate=60, interlace=false, no_sdi=true, four_k_only=true }
}

-- NTSC timing formats (time divided by 1.001)
_M.specialFormatMap = {
	["1080P23"] = "1080P24",
	["1080P23.98"] = "1080P24",
	["1080P29"] = "1080P30",
	["1080P29.97"] = "1080P30",
	["1080P59"] = "1080P60",
	["1080P59.94"] = "1080P60",
	["720P59"] = "720P60",
	["720P59.94"] = "720P60",
	["1080I59"] = "1080I60",
	["1080I59.94"] = "1080I60",
	["480I59"] = "NTSC",
	["480I59.94"] = "NTSC",
	["480P59"] = "480P60",
	["480P59.94"] = "480P60"
}

_M.reverseSpecialFormatMap = {
	["1080P24"] = "1080P23",
	["1080P30"] = "1080P29",
	["1080P60"] = "1080P59",
	["720P60"] = "720P59",
	["1080I60"] = "1080I59",
	["NTSC"] = "480I59",
	["480P60"] = "480P59",
}

----------------------------------------------

function _M.Init()
	hi_hdmi.Init()
	kv_sdi.Init()
	_M.voDrivers["HDMI"] = hi_hdmi.GetInstance(0)
	_M.voDrivers["BT1120"] = kv_sdi.GetInstance(0)
	_M.voDrivers["SDI"] = _M.voDrivers["BT1120"]
	vodrv_common.Init(_M)
end

--
_M.FetchBestFormat = vodrv_common.FetchBestFormat
_M.CheckSupportedFormats = vodrv_common.CheckSupportedFormats
_M.GetSyncDetail = vodrv_common.GetSyncDetail
_M.GetNearestResolution = vodrv_common.GetNearestResolution

--
function _M.GetSoCSupportDevices()
	return _M.socSupports
end

--
function _M.GetSupportInterfaces()
	return _M.voSupports
end

--
function _M.CheckInterfaceUsing(ifname)
	local x = _M.voSupports[ifname or "invalid"]
	if not x then
		return "#INVALID#"
	end
	return _M.voUsing[ x.ifname ]
end

--
function _M.MarkInterfaceUsing(ifname,user)
	local x = _M.voSupports[ifname or "invalid"]
	if not x then
		return 
	end

	if user then
		_M.voUsing[x.ifname] = user
	else
		_M.voUsing[x.ifname] = nil
	end
end

--
function _M.GetInterfaceDriver(ifname)
	if not ifname then return nil end
	return _M.voDrivers[ifname]
end

return _M

