-- Downloader dir mapping, for different products, you can
-- replace this file to your mapping.

local client = require "coaf.client"
local _dbg = require "coaf.debug"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)

local proxy_Cli

local function getSysctrlProxy()
	local proxy
	if not proxy_Cli then
		proxy_Cli = client.New( "systemctrl", "local" )
		if not proxy_Cli then
			_dbg.Error( "DOWNLOAD", "Fail to create systemctrl client")
			return nil
		end
	end

	proxy = proxy_Cli:CreateProxy("/")
	if not proxy then
		_dbg.Error( "DOWNLOAD", "Fail to create co-processor systemctrl proxy /" )
		return nil
	end
	return proxy
end

local _M = {

["firmware"] = { 
	location = "/tmp",
	-- You can add a function named 'filterUrl' to modify URL before download:
	filterUrl = function(url)
		if not url then
			return nil
		else
			local flag = url:match("^(.*)://.*")
			if flag then
				return url
			else
				local host = "192.168.0.7"
				local trunk,err = loadfile( "/data/configs/platform.conf" )
				if trunk then
					local done, addr, pport, wport = pcall(trunk)
					if done and addr then
						host = addr .. ":" .. (wport or 80)
					end
				else
					_dbg.Error( "DOWNLOAD", "Fail to load platform configure file:", err )
				end
				return "http://" .. host .. url
			end
		end
	end,
	--
	-- You can add a function named 'beforeProcess' to prepare downloading:
	beforeProcess = function(dir,file,url)
		os.execute( "/bin/mkdir -p " .. dir )
		os.execute( "/bin/rm -Rf " .. file )
		os.execute( "/bin/rm -Rf /tmp/.*" ) --Clear the exist temporary files.
		return true
	end,
	--
	-- You can add a functin named 'afterProcess' to process downloaded file:
	afterProcess = function(status,dir,file)
		local sysctrl = getSysctrlProxy()
		if not sysctrl then
			_dbg.Error( "DOWNLOAD", "Fail to get systemctrl proxy!" )
			return false, "Fail to get systemctrl proxy"
		else
			local r,reason,err = sysctrl.ProcessFirmware( dir .. "/" .. file )
			os.execute( "/bin/rm -Rf " )
			if not r or not reason then
				sysctrl:Destroy()
				return false, reason
			else
				-- sysctrl.Reboot(3) -- **DONT** After 3 seconds later, reboot.
				sysctrl:Destroy()
				return true
			end
		end
	end
},

["font"] = { 
	location = "/usr/share/fonts",
	filterUrl = function(url)
		if not url then
			return nil
		else
			local flag = url:match("^(.*)://.*")
			if flag then
				return url
			else
				local host = "192.168.0.7"
				local trunk = loadfile( "/data/configs/platform.conf" )
				if trunk then
					local done, addr, pport, wport = pcall(trunk)
					if done and addr then
						host = addr .. ":" .. (wport or 80)
					end
				end
				return "http://" .. host .. url
			end
		end
	end
}

}

return _M
