-------------------------------------------------
-- Network-manager static route
-------------------------------------------------
local coaf = require "coaf.core"
local thread = require "coaf.thread"
local _dbg = require "coaf.debug"
local lfs = require "lfs"
local staticroutecfg = require "staticRoutecfg"
local _M = {
    staticList = nil,
    static_seed = 1,
}

------------------------------------------------

function _M.Init()
    _M.staticList = staticroutecfg.Load()
    _M.static_seed = 1
    local k,v
    for k,v in pairs(_M.staticList) do
        if k>_M.static_seed then
            _M.static_seed = k
        end
    end

end

function copy_merge(target,...)
	target = target or {}
	local items={...}
	local i
	for i=1, #items do
		if type(items[i]) == "table" then
			local k, v
			for k,v in pairs(items[i]) do
				target[k] = v
			end
		end
	end
	return target
end
local function maskCheckByte(b)
	local cc
	local r = 0
	for cc = 1, 8, 1 do
		if (b & 0x80) > 0 then
			r = r + 1
		else
			return r
		end
		b = b << 1
	end
	return r
end

function _M.ip4MaskToPrefix(mask)
	if not mask or mask == "" then
		return 0
	else
		local t1,t2,t3,t4 = mask:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")
		local r = 0
		t1 = tonumber(t1)
		t2 = tonumber(t2)
		t3 = tonumber(t3)
		t4 = tonumber(t4)

		if t1 == 255 then
			r = 8
			if t2 == 255 then
				r = r + 8
				if t3 == 255 then
					r = r + 8 + maskCheckByte(t4)
				else
					r = r + maskCheckByte(t3)
				end
			else
				r = r + maskCheckByte(t2)
			end
		else
			r = r + maskCheckByte(t1)
		end

		return r
	end
end


function _M.AddStatic(details)
    if not details then
        return false,"not args"
    end
    for k,v in pairs(_M.staticList) do
        if details.interface == v.interface and details.target == v.target and details.netmask == v.netmask and details.gateway == v.gateway and details.mtu == v.mtu and details.metric == v.metric then
            return false,"The current static route already exists"
        end
    end
    local mask = _M.ip4MaskToPrefix(details.netmask)
    t = io.popen("ip route add " .. details.target .. '/' .. mask .. " via " .. details.gateway .. " dev " .. details.interface .. " metric " .. details.metric .. " mtu " .. details.mtu ..  " 2>&1")
    if t then
        local res = t:read("*all")
        t:close()
        if #res>0 then
            local err = string.gsub(res,"\n","")
            return false,err
        end
        details['status'] = 1
        _M.static_seed =_M.static_seed + 1
        _M.staticList[_M.static_seed] = details
        _M.staticList:__TRIGGER(true)
        return true
    else
        return false
    end
end

function _M.GetStatic()
    result = {}
    for k,v in pairs(_M.staticList) do
        table.insert( result, copy_merge({id=k},v) )
    end
    table.sort(result,function(a,b)
        return a.id<b.id
    end)
    return result
end

function _M.ModifyStatic(id,details)
    local exist_detail = _M.staticList[tonumber(id)]
    if not exist_detail then
        return false,"Static route does not exist"
    else
        for k,v in pairs(_M.staticList) do
            if details.interface == v.interface and details.target == v.target and details.netmask == v.netmask and details.gateway == v.gateway and details.mtu == v.mtu and details.metric == v.metric then
                return false,"The current static route already exists"
            end
        end
        if exist_detail.status == 0 then
            copy_merge(exist_detail,details)
            _M.staticList:__TRIGGER(true)
            return true
        else
            _M.SetStatic(tonumber(id),0) -- 停止
            local mask = _M.ip4MaskToPrefix(details.netmask)
            t = io.popen("ip route add " .. details.target .. '/' .. mask .. " via " .. details.gateway .. " dev " .. details.interface .. " metric " .. details.metric .. " mtu " .. details.mtu ..  " 2>&1")
            if t then
                local res = t:read("*all")
                t:close()
                if #res>0 then
                    local err = string.gsub(res,"\n","")
                    _M.SetStatic(tonumber(id),1) -- 开启  修改失败，重新开启修改前的
                    return false,err
                end
                details['status'] = 1
                copy_merge(exist_detail,details)
                _M.staticList:__TRIGGER(true)
                return true
            else
                return false
            end
        end
    end
    return true
end

function _M.RemoveStatic(ids)
    if type(ids) == "string" or type(ids) == "number" then
        local id = tonumber(ids)
        _M.SetStatic(id,0)
        _M.staticList[id] = nil
    elseif type(ids) == "table" then
        local i,id
        for i=1,#ids do
            id = tonumber(ids[i])
            _M.SetStatic(id,0)
            _M.staticList[id] = nil
        end
    else
        return false,"无效id"
    end
    _M.staticList:__TRIGGER(true)
    return true
end

function _M.SetStatic(id,status)
    local exist_detail = _M.staticList[tonumber(id)]
    if not exist_detail then
        return false,"Static route does not exist"
    else
        if status == exist_detail.status then
        -- copy_merge(exist_detail,details)
        -- _M.staticList:__TRIGGER(true)
            return true
        else
            local t = true
            local mask = _M.ip4MaskToPrefix(exist_detail.netmask)
            if not mask then
                t = false
            elseif status == 1 then
                t = io.popen("ip route add " .. exist_detail.target .. '/' .. mask .. " via " .. exist_detail.gateway .. " dev " .. exist_detail.interface .. " metric " .. exist_detail.metric .. " mtu " .. exist_detail.mtu ..  " 2>&1")
            elseif status == 0 then
                t = io.popen("ip route del " .. exist_detail.target .. '/' .. mask .. " via " .. exist_detail.gateway .. " dev " .. exist_detail.interface .. " metric " .. exist_detail.metric .. " mtu " .. exist_detail.mtu ..  " 2>&1")
            end
            if t then
                local res = t:read("*all")
                t:close()
                if #res>0 then
                    local err = string.gsub(res,"\n","")
                    return false,err
                end
                exist_detail["status"] = status
                _M.staticList:__TRIGGER(true)
                return true
            else
                return false
            end
        end
    end
end

function _M.Start(iframe)
    -- for k,v in pairs(_M.staticList) do
    --     os.execute("route del -net " .. v.target .. ' netmask ' .. v.netmask .. " gw " .. v.gateway .. " dev " .. v.interface)
    -- end
    for k,v in pairs(_M.staticList) do
        if v.interface == iframe then
            local mask = _M.ip4MaskToPrefix(v.netmask)
            if v.status == 1 then
                local t = io.popen("ip route add " .. v.target .. '/' .. mask .. " via " .. v.gateway .. " dev " .. v.interface .. " metric " .. v.metric .. " mtu " .. v.mtu ..  " 2>&1")
                if t then
                    local res = t:read("*all")
                    t:close()
                    if #res>0 then
                        v.status = 0
                    end
                else
                    v.status = 0            
                end
            end
        end
    end
    _M.staticList:__TRIGGER(true)
end


return _M