#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local CO = require "mod_coservice"
local pt = require "pl.pretty"

require "himpp"
require "osd2"
local mo_OSD = require "mod_ImgOSD"
local mo_OSD_adap = require "mod_OSD_adapter"

_G.osdcfg = require("osd_config").Load()

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/osd.log")
	if not logF or not logF:init() then
		_dbg.Error("OSD", "Fail to open log file: /var/log/osd.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

local for50msTimer

local osd_update_counter = 0
local osd_init_check_count = 0
local for50ms_timer_working = false
local function for50ms_PeriodCheck()
	osd_update_counter = osd_update_counter + 1
	osd_init_check_count = osd_init_check_count + 1
	--Don't enter twice!
	if for50ms_timer_working then
		return
	end
	for50ms_timer_working = true
	if osd_update_counter >= 10 then --Aka 500ms.
		osd_update_counter = 0
		mo_OSD_adap.PeriodRender()
	end

	if osd_init_check_count >= 600 then --Aka 30s.
		osd_init_check_count = 0
		mo_OSD.Check()
	end

	for50ms_timer_working = false
end

--------------------------------------

coaf.Init( "osd_service" )

service.InitSocket()
client.InitSocket( socket, adapter )

local cosvc_osd

local function Main(args)

    cosvc_osd = service.New( "OSD", "unix:/var/run/osd.sock" )
	if cosvc_osd then
		_dbg.Info( "OSD", "CoService of 'osd' is created." )
		cosvc_osd:CreateCoServerProxy()
		cosvc_osd:ExposeObject("/osd", mo_OSD )
	else
		_dbg.Error( "OSD", "Fail created CoService!" )
    end
    
	mo_OSD.Init()
	
	for50msTimer = coaf.NewTimer( for50ms_PeriodCheck, 0.01, 0.05 )
	if for50msTimer then
		for50msTimer:start()
	end
end

coaf.Start(Main, arg)

if for50msTimer then
	for50msTimer:stop()
end

mo_OSD.Destroy()

_dbg.Print( "OSD", "Service terminated." )

_dbg.Print( "OSD", "===Patch: kill self" )
local posix = require "posix"
local signal = require "posix.signal"
local pid_info = posix.getpid()
local pid = pid_info.pid
signal.kill( pid, 1 )
_dbg.Print( "OSD", "===Patch: If you can see this, self killing is fail!" )
