#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local event = require "coaf.event"

local withLog = false
local _x
for _x=1, #arg do
        if arg[_x] == "--log" then
                withLog = true
                break
        end
end

if withLog then
        -- ** initialize the log file **
        local roll = require "coaf.log.rollfile"
        os.execute( "/bin/mkdir -p /var/log/" )
        local logF = roll.Create("/var/log/picManager.log")
        if not logF or not logF:init() then
                _dbg.Error("picManager", "Fail to open log file: /var/log/picManager.log, so the log messages should be output to console.")
                withLog = false
        else
                _dbg.AddLogger("logfile", logF, nil, nil)
        end
        -- ** log **
end

coaf.Init( "picManager" )
if withLog then
    local stdio = require "coaf.log.stdio"
    stdio.SetDefaultFilterName( "picManager" )
    stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

_G.cfg_pics = require("configs.pics_config").Load()
_G.cfg_imgRelation = require("configs.imgRelation_config").Load()

--todo


_G.g_event = event.createNew( "picManager" )

local mod_picManager = require "mod_picManager"
local mod_relationImg = require "mod_relationImg"

local cosvc_picManager
------------------
mod_picManager.init()
mod_relationImg.GetHomePicInfos()
os.execute( "/bin/mkdir -p /data/storage/img"  )
os.execute("/bin/chmod 777 -R /data/storage/img")

-- **Main entry**
local function Main(args)
        cosvc_picManager = service.New( "picManager", "unix:/var/run/picManager.sock" )
        if cosvc_picManager then
                _dbg.Info( "picManager", "CoService of picManager is created." )
                cosvc_picManager:CreateCoServerProxy()
                cosvc_picManager:ExposeObject("/pics", mod_picManager )
                cosvc_picManager:ExposeObject("/relation", mod_relationImg )
        else
                _dbg.Error( "picManager", "Fail created CoService!" )
        end

end

coaf.Start(Main, arg)

_dbg.Info("picManager", "picManager is destroying." )
event.free( "picManager" )
_dbg.Print( "picManager", "Service terminated." )





