local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local hiperiphery = require "hiperiphery"
local I2C = hiperiphery.I2C

local _M = {
	I2C_DEV = "/dev/i2c-0",
	BASE_ADDR = 0xa4,
	SAVED_VERSION = nil
}

function _M.ReadReg( addr, itemArr )
	local done,err = pcall( function() _M.i2c_handle = I2C( _M.I2C_DEV ) end )
	if not done then
		if type(err) == "table" then
			_dbg.Error( "HW", "Could not open I2C device for FPGA controlling. ErrMsg:", err.message or "---" )
		else
			_dbg.Error( "HW", "Could not open I2C device for FPGA controlling. ErrMsg:", err )
		end
		return nil
	else
		_dbg.Info( "HW", "FPGA controller openned, at " .. string.format("%02x",_M.BASE_ADDR) .. "@" .. _M.I2C_DEV )
	end

	if not itemArr or type(itemArr) ~= "table" then
		itemArr = { 0xff, flags = I2C.I2C_M_RD }
	else
		if #itemArr < 1 then
			itemArr[1] = 0xff
		end
		if not itemArr.flags then
			itemArr.flags = I2C.I2C_M_RD
		else
			itemArr.flags = itemArr.flags | I2C.I2C_M_RD
		end
	end

	local addrs = {}
	local i
	for i=1, #itemArr do
		addrs[i] = addr+i-1
	end

	local msg = { addrs, itemArr }
	local done,err = pcall( function() _M.i2c_handle:transfer( _M.BASE_ADDR>>1, msg ) end )
	_M.i2c_handle:close()
	if not done then
		if type(err) == "table" then
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err.message or "---" )
			return nil, err.message or "---"
		else
			--print( string.format( "ReadReg fail: 0x%02x [0x%02x], Error:", BASE_ADDR, addr), err )
			return nil, err
		end
	else
		return itemArr
	end
end


-----------------------------------

function _M.GetHardwareVersion()
	if _M.SAVED_VERSION then
		return _M.SAVED_VERSION
	else
		local ret = _M.ReadReg( 0xf0, {0x00} )
		if not ret then
			return "2.0.00"
		else
			_M.SAVED_VERSION = "2.0." .. string.format( "%02X", ret[1] )
			return _M.SAVED_VERSION
		end
	end
end

-----------------------------------

return _M


