local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local Date = require 'pl.Date'
local pl = require 'pl.pretty'
local _dbg = require 'coaf.debug'
adapter.timeoutUnit = 1000
client.InitSocket(nil, adapter)
pcall(require, 'lsystime')

local _M = {}

function _M.GetProxy(objPath, service, loc, timeout, addr)
	local proxy
	local cli

	cli = client.New( service, loc, addr, timeout or 3 ) --3 seconds timeout.
	if not cli then
		_dbg.Error( "HWCLOCK", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		_dbg.Error( "HWCLOCK", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
    end
	return proxy
end


function _M.Load(full)
	-- os.execute( "/sbin/hwclock -s" ) -- set the system time from the RTC
	
	local proxy = _M.GetProxy( "/", "rtc" )
	if not proxy then
		return false, "无法连接rtc服务"
    else
        local r, done,err = proxy.Load()
        proxy:Destroy()
        if not r or not done then
            _dbg.Error("HWCLOCK", "从RTC同步时间失败!!!! err:",err)
			return false,"从RTC同步时间失败"
		else
			_dbg.Info("HWCLOCK", "同步RTC时间成功!!!")
			return true
        end
    end
end

function _M.Store(full)
	-- os.execute( "/sbin/hwclock -w" ) -- set the RTC from the system time
	-- local datetime = os.date("%Y-%m-%d %H:%M:%S", os.time())
	-- _dbg.Info("datetime=",)

	local proxy = _M.GetProxy( "/", "rtc" )
	if not proxy then
		return false, "无法连接rtc服务"
    else
        local r, done,err = proxy.Store()
        proxy:Destroy()
        if not r or not done then
            _dbg.Error("HWCLOCK", "设置RTC时间失败!!!!err:",err)
			return false,"设置RTC时间失败"
		else
			_dbg.Info("HWCLOCK", "设置RTC时间成功!!!!")
			return true
        end
    end
end

return _M

