local coaf = require "coaf.core"
local thread = require "coaf.thread"
local _dbg = require "coaf.debug"
local TOOLS = require "system.cmdtools"

local _M = {
	running_pid = nil,
	running_task = nil
}

local function waitTask(thrObj)
	_dbg.Info( "SYSCTRL", "NTP server is running..." )
	local r,flag = coaf.WaitPID(_M.running_pid)
	if not r then
		local st, signal = pcall(require, "posix.signal")
		if st and signal then
			signal.kill( _M.running_pid )
		end
	end
	_dbg.Info( "SYSCTRL", "NTP server is terminated." )
	_M.running_task = nil
	_M.running_pid = nil
end

function _M.Start()
	if _M.running_task then
		return true
	end

	_M.running_pid = coaf.Fork( function()
		local st, posix_unistd = pcall(require,"posix.unistd")
		if not st or not posix_unistd then
			os.exit(1)
		else
			local args
			if type(TOOLS.NTP_SVR_ARGS) == "function" then
				args = TOOLS.NTP_SVR_ARGS(servers)
			elseif type(TOOLS.NTP_SVR_ARGS) == "table" then
				args = TOOLS.NTP_SVR_ARGS
			else
				args = servers
			end
			posix_unistd.execp( TOOLS.NTP_SVR, args )
			os.exit(0)
		end
	end)

	if _M.running_pid then
		_M.running_task = thread.New(waitTask)
		_M.running_task:Start()
		return true
	else
		return false
	end

	return true
end

function _M.Stop()
	if _M.running_task then
		_M.running_task:Terminate()
	end
end

return _M

